from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible.module_utils.common.text.converters import to_native

from ansible_collections.ansible.mysql.plugins.module_utils.mysql import (
    get_server_implementation,
    get_server_version,
)


def get_server_version_tuple(cursor):
    version = get_server_version(cursor).split('-', 1)[0]
    version_tuple = []

    for part in version.split('.'):
        digits = ''.join(char for char in part if char.isdigit())
        if not digits:
            break
        version_tuple.append(int(digits))

    while len(version_tuple) < 3:
        version_tuple.append(0)

    return tuple(version_tuple[:3])


def ensure_resource_groups_supported(module, cursor):
    if get_server_implementation(cursor) != 'mysql':
        module.fail_json(msg='Resource groups are supported only by MySQL 8.0 or later.')

    if get_server_version_tuple(cursor) < (8, 0, 0):
        module.fail_json(msg='Resource groups are supported only by MySQL 8.0 or later.')

    try:
        cursor.execute("SELECT RESOURCE_GROUP_NAME FROM INFORMATION_SCHEMA.RESOURCE_GROUPS LIMIT 1")
        cursor.fetchall()
    except Exception as e:
        module.fail_json(msg='Resource groups are not supported by the server: %s' % to_native(e))


def normalize_vcpu_ids(vcpu_ids):
    if vcpu_ids is None:
        return None
    if isinstance(vcpu_ids, bytes):
        vcpu_ids = vcpu_ids.decode('utf-8')

    cpus = set()

    for raw_item in vcpu_ids.split(','):
        item = raw_item.strip()
        if not item:
            raise ValueError('vcpu_ids contains an empty element')

        if '-' in item:
            bounds = item.split('-', 1)
            if len(bounds) != 2 or not bounds[0] or not bounds[1]:
                raise ValueError('vcpu_ids contains an invalid range: %s' % item)
            start = int(bounds[0])
            end = int(bounds[1])
            if start > end:
                raise ValueError('vcpu_ids contains a descending range: %s' % item)
            cpus.update(range(start, end + 1))
        else:
            cpus.add(int(item))

    normalized = []
    sorted_cpus = sorted(cpus)
    range_start = None
    range_end = None

    for cpu in sorted_cpus:
        if range_start is None:
            range_start = cpu
            range_end = cpu
            continue

        if cpu == range_end + 1:
            range_end = cpu
            continue

        normalized.append(_format_cpu_range(range_start, range_end))
        range_start = cpu
        range_end = cpu

    if range_start is not None:
        normalized.append(_format_cpu_range(range_start, range_end))

    return ','.join(normalized)


def _format_cpu_range(start, end):
    if start == end:
        return str(start)
    return '%s-%s' % (start, end)


def normalize_resource_group_enabled(value):
    if isinstance(value, bool):
        return value
    return str(value).upper() in ('YES', '1', 'ON', 'TRUE', 'ENABLE', 'ENABLED')


def normalize_resource_group_row(row):
    return {
        'name': row['RESOURCE_GROUP_NAME'],
        'resource_group_type': row['RESOURCE_GROUP_TYPE'],
        'enabled': normalize_resource_group_enabled(row['RESOURCE_GROUP_ENABLED']),
        'vcpu_ids': normalize_vcpu_ids(row['VCPU_IDS']),
        'thread_priority': int(row['THREAD_PRIORITY']),
    }


def validate_resource_group_type(resource_group_type):
    normalized_type = resource_group_type.upper()
    if normalized_type not in ('SYSTEM', 'USER'):
        raise ValueError('resource_group_type must be SYSTEM or USER')
    return normalized_type


def validate_thread_priority(resource_group_type, thread_priority):
    if thread_priority < -20 or thread_priority > 19:
        raise ValueError('thread_priority must be between -20 and 19')

    if resource_group_type == 'SYSTEM' and thread_priority > 0:
        raise ValueError('thread_priority must be between -20 and 0 for SYSTEM resource groups')

    if resource_group_type == 'USER' and thread_priority < 0:
        raise ValueError('thread_priority must be between 0 and 19 for USER resource groups')

    return thread_priority



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_mysqlconfiguration_info
version_added: "0.1.2"
short_description: Get Azure MySQL Configuration facts
description:
    - Get facts of Azure MySQL Configuration.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource. You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    server_name:
        description:
            - The name of the server.
        required: True
        type: str
    name:
        description:
            - Setting name.
        type: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Get specific setting of MySQL Server
  azure_rm_mysqlconfiguration_info:
    resource_group: myResourceGroup
    server_name: testmysqlserver
    name: deadlock_timeout

- name: Get all settings of MySQL Server
  azure_rm_mysqlconfiguration_info:
    resource_group: myResourceGroup
    server_name: server_name
'''

RETURN = '''
settings:
    description:
        - A list of dictionaries containing MySQL Server settings.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Setting resource ID.
            returned: always
            type: str
            sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.DBforMySQL/servers/testmysqlser
                     ver/configurations/deadlock_timeout"
        name:
            description:
                - Setting name.
            returned: always
            type: str
            sample: deadlock_timeout
        value:
            description:
                - Setting value.
            returned: always
            type: raw
            sample: 1000
        description:
            description:
                - Description of the configuration.
            returned: always
            type: str
            sample: Deadlock timeout.
        source:
            description:
                - Source of the configuration.
            returned: always
            type: str
            sample: system-default
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMySqlConfigurationInfo(AzureRMModuleBase):
    def __init__(self):
        # define user inputs into argument
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            server_name=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str'
            )
        )
        # store the results of the module operation
        self.results = dict(changed=False)
        self.resource_group = None
        self.server_name = None
        self.name = None
        super(AzureRMMySqlConfigurationInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results['settings'] = self.get()
        else:
            self.results['settings'] = self.list_by_server()
        return self.results

    def get(self):
        '''
        Gets facts of the specified MySQL Configuration.

        :return: deserialized MySQL Configurationinstance state dictionary
        '''
        response = None
        results = []
        try:
            response = self.mysql_client.configurations.get(resource_group_name=self.resource_group,
                                                            server_name=self.server_name,
                                                            configuration_name=self.name)
            self.log("Response : {0}".format(response))
        except ResourceNotFoundError as e:
            self.log('Could not get facts for Configurations.')

        if response is not None:
            results.append(self.format_item(response))

        return results

    def list_by_server(self):
        '''
        Gets facts of the specified MySQL Configuration.

        :return: deserialized MySQL Configurationinstance state dictionary
        '''
        response = None
        results = []
        try:
            response = self.mysql_client.configurations.list_by_server(resource_group_name=self.resource_group,
                                                                       server_name=self.server_name)
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Could not get facts for Configurations.')

        if response is not None:
            for item in response:
                results.append(self.format_item(item))

        return results

    def format_item(self, item):
        d = item.as_dict()
        d = {
            'resource_group': self.resource_group,
            'server_name': self.server_name,
            'id': d['id'],
            'name': d['name'],
            'value': d['value'],
            'description': d['description'],
            'source': d['source']
        }
        return d


def main():
    AzureRMMySqlConfigurationInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import secrets
import string
from typing import TypedDict

# Length of random marker strings used for output parsing
MARK_LENGTH = 26

# Exit code returned when S3 download fails in Windows executor error handler
S3_DOWNLOAD_ERROR_EXIT_CODE = 99


def generate_mark() -> str:
    """Generates a random string of characters to delimit SSM CLI commands."""
    return "".join([secrets.choice(string.ascii_letters) for _i in range(MARK_LENGTH)])


class CommandResult(TypedDict):
    """
    A dictionary that contains the executed command results.
    """

    returncode: int
    stdout: str
    stderr: str



#!/usr/bin/python
#
# Copyright (c) 2021 Aparna Patil(@techcon65)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_proximityplacementgroup_info

version_added: "1.6.0"

short_description: Get proximity placement group facts

description:
    - Get facts for specified proximity placement group or all proximity placement groups in a given resource group.

options:
    resource_group:
        description:
            - Name of resource group.
        type: str
    name:
        description:
            - The name of the proximity placement group.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Aparna Patil (@techcon65)

'''

EXAMPLES = '''
- name: Get facts for one proximity placement group
  azure_rm_proximityplacementgroup_info:
    resource_group: myAzureResourceGroup
    name: myppg

- name: Get facts for all proximity placement groups in resource group
  azure_rm_proximityplacementgroup_info:
    resource_group: myAzureResourceGroup
'''

RETURN = '''
proximityplacementgroups:
    description:
        - Gets a list of proximity placement groups.
    returned: always
    type: list
    elements: dict
    sample: [
        {
            "availability_sets": [
                {
                    "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/
                           providers/Microsoft.Compute/availabilitySets/availabilityset1"
                },
                {
                    "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/
                           providers/Microsoft.Compute/availabilitySets/availabilityset2"
                }
            ],
            "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/providers/
                   Microsoft.Compute/proximityPlacementGroups/myppg",
            "location": "eastus",
            "name": "myppg",
            "proximity_placement_group_type": "Standard",
            "tags": {},
            "virtual_machine_scale_sets": [],
            "virtual_machines": [
                {
                    "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myAzureResourceGroup/
                           providers/Microsoft.Compute/virtualMachines/mylinuxvm"
                }
            ]
        }
    ]
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

AZURE_OBJECT_CLASS = 'ProximityPlacementGroup'


class AzureRMProximityPlacementGroupInfo(AzureRMModuleBase):

    def __init__(self):

        # define user inputs into argument
        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        # store the results of the module operation
        self.results = dict(
            changed=False
        )

        self.name = None
        self.resource_group = None
        self.tags = None

        super(AzureRMProximityPlacementGroupInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=False, facts_module=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        results = []
        # list the conditions and results to return based on user input
        if self.name is not None:
            # if there is a group name provided, return facts about that specific proximity placement group
            results = self.get_item()
        elif self.resource_group:
            # all the proximity placement groups listed in specific resource group
            results = self.list_resource_group()
        else:
            # all the proximity placement groups in a subscription
            results = self.list_items()

        self.results['proximityplacementgroups'] = self.curated_items(results)

        return self.results

    def get_item(self):
        self.log('Get properties for {0}'.format(self.name))
        item = None
        results = []
        # get specific proximity placement group
        try:
            item = self.compute_client.proximity_placement_groups.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass

        # serialize result
        if item and self.has_tags(item.tags, self.tags):
            results = [item]
        return results

    def list_resource_group(self):
        self.log('List all proximity placement groups for resource group - {0}'.format(self.resource_group))
        try:
            response = self.compute_client.proximity_placement_groups.list_by_resource_group(self.resource_group)
        except ResourceNotFoundError as exc:
            self.fail("Failed to list for resource group {0} - {1}".format(self.resource_group, str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(item)
        return results

    def list_items(self):
        self.log('List all proximity placement groups for a subscription ')
        try:
            response = self.compute_client.proximity_placement_groups.list_by_subscription()
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items - {0}".format(str(exc)))

        results = []
        for item in response:
            if self.has_tags(item.tags, self.tags):
                results.append(item)
        return results

    def curated_items(self, raws):
        return [self.ppg_to_dict(item) for item in raws] if raws else []

    def ppg_to_dict(self, ppg):
        result = dict(
            id=ppg.id,
            name=ppg.name,
            location=ppg.location,
            tags=ppg.tags,
            proximity_placement_group_type=ppg.proximity_placement_group_type,
            virtual_machines=[dict(id=x.id) for x in ppg.virtual_machines],
            virtual_machine_scale_sets=[dict(id=x.id) for x in ppg.virtual_machine_scale_sets],
            availability_sets=[dict(id=x.id) for x in ppg.availability_sets]
        )
        return result


def main():
    AzureRMProximityPlacementGroupInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The nxos interfaces fact class
It is in this file the configuration is collected from the device
for a given resource, parsed, and the facts tree is populated
based on the configuration.
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common import (
    utils,
)

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.interfaces.interfaces import (
    InterfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.rm_templates.interfaces import (
    InterfacesTemplate,
)


class InterfacesFacts(object):
    """The nxos interfaces facts class"""

    def __init__(self, module, subspec="config", options="options"):
        self._module = module
        self.argument_spec = InterfacesArgs.argument_spec

    def _get_interface_config(self, connection):
        return connection.get("show running-config | section ^interface")

    def populate_facts(self, connection, ansible_facts, data=None):
        """Populate the facts for Interfaces network resource

        :param connection: the device connection
        :param ansible_facts: Facts dictionary
        :param data: previously collected conf

        :rtype: dictionary
        :returns: facts
        """
        facts = {}
        objs = []

        if not data:
            data = self._get_interface_config(connection)

        # parse native config using the Interfaces template
        interfaces_parser = InterfacesTemplate(lines=data.splitlines(), module=self._module)
        objs = list(interfaces_parser.parse().values())
        ansible_facts["ansible_network_resources"].pop("interfaces", None)
        facts = {"interfaces": []}
        params = utils.remove_empties(
            interfaces_parser.validate_config(self.argument_spec, {"config": objs}, redact=True),
        )

        facts["interfaces"] = params["config"]
        ansible_facts["ansible_network_resources"].update(facts)

        return ansible_facts



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_channel_utilization_history_info
short_description: Information module for networks _wireless _channel _utilization
  _history
description:
  - Get all networks _wireless _channel _utilization _history.
  - Return AP channel utilization over time for a device or network client.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
      - NetworkId path parameter. Network ID.
    type: str
  t0:
    description:
      - T0 query parameter. The beginning of the timespan for the data. The maximum
        lookback period is 31 days from today.
    type: str
  t1:
    description:
      - T1 query parameter. The end of the timespan for the data. T1 can be a maximum
        of 31 days after t0.
    type: str
  timespan:
    description:
      - >
        Timespan query parameter. The timespan for which the information will be fetched.
        If specifying timespan, do not specify parameters t0 and t1. The value must
        be in seconds and be less than or equal to 31 days. The default is 7 days.
    type: float
  resolution:
    description:
      - >
        Resolution query parameter. The time resolution in seconds for returned data.
        The valid resolutions are 600, 1200, 3600, 14400, 86400. The default is 86400.
    type: int
  autoResolution:
    description:
      - >
        AutoResolution query parameter. Automatically select a data resolution based
        on the given timespan; this overrides the value specified by the 'resolution'
        parameter. The default setting is false.
    type: bool
  clientId:
    description:
      - >
        ClientId query parameter. Filter results by network client to return per-device,
        per-band AP channel utilization metrics inner joined by the queried client's
        connection history.
    type: str
  deviceSerial:
    description:
      - >
        DeviceSerial query parameter. Filter results by device to return AP channel
        utilization metrics for the queried device; either band or clientId must be
        jointly specified.
    type: str
  apTag:
    description:
      - >
        ApTag query parameter. Filter results by AP tag to return AP channel utilization
        metrics for devices labeled with the given tag; either clientId or deviceSerial
        must be jointly specified.
    type: str
  band:
    description:
      - Band query parameter. Filter results by band (either '2.4', '5' or '6').
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless getNetworkWirelessChannelUtilizationHistory
    description: Complete reference of the getNetworkWirelessChannelUtilizationHistory
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-network-wireless-channel-utilization-history
notes:
  - SDK Method used are
    wireless.Wireless.get_network_wireless_channel_utilization_history,
  - Paths used are
    get /networks/{networkId}/wireless/channelUtilizationHistory,
"""

EXAMPLES = r"""
- name: Get all networks _wireless _channel _utilization _history
  cisco.meraki.networks_wireless_channel_utilization_history_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    t0: string
    t1: string
    timespan: 0
    resolution: 0
    autoResolution: true
    clientId: string
    deviceSerial: string
    apTag: string
    band: string
    networkId: string
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "startTs": "string",
        "endTs": "string",
        "utilizationTotal": 0,
        "utilization80211": 0,
        "utilizationNon80211": 0
      }
    ]
"""



# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import annotations

import time
import typing
from functools import wraps

if typing.TYPE_CHECKING:
    from typing import Any
    from typing import Callable

from ansible_collections.amazon.aws.plugins.module_utils.cloud import BackoffIterator


class AWSConnectionRetry:
    """
    Decorator class for retrying connection plugin operations with exponential backoff.

    Designed for use with AWS connection plugins that need to restart connections
    on failure (e.g., SSM, which needs to close and reopen sessions).

    The decorated method must be an instance method on an object with:
    - reconnection_retries (int): Number of retry attempts
    - verbosity_display (callable): Function for logging verbosity messages
    - close() (optional): Method to close/restart the connection

    Example usage:
        @AWSConnectionRetry.exponential_backoff()
        def exec_command(self, cmd):
            # command execution code
            pass
    """

    @staticmethod
    def _get_max_retries(instance: Any, retries_param: int | None) -> int:
        """Get the maximum retry count from parameter or instance attribute."""
        if retries_param is not None:
            return retries_param
        return int(getattr(instance, "reconnection_retries", 0))

    @staticmethod
    def _get_cmd_summary(args: tuple) -> str:
        """Extract command summary from arguments for logging."""
        if args:
            return f"{args[0]}..."
        return "command"

    @staticmethod
    def _get_display_func(instance: Any) -> Callable:
        """Get verbosity display function or return no-op."""
        return getattr(instance, "verbosity_display", lambda level, msg: None)

    @staticmethod
    def _restart_connection(instance: Any) -> None:
        """Close and restart connection if close method exists."""
        if hasattr(instance, "close"):
            instance.close()

    @staticmethod
    def _format_retry_message(
        attempt: int, max_retries: int, exception: Exception, cmd_summary: str, pause: float
    ) -> str:
        """Format the retry log message."""
        return (
            f"AWSConnectionRetry: attempt {attempt}/{max_retries + 1}, "
            f"caught exception ({type(exception).__name__}: {exception}) "
            f"from cmd ({cmd_summary}), pausing for {pause} seconds"
        )

    @classmethod
    def exponential_backoff(
        cls,
        retries: int | None = None,
        initial_delay: int = 1,
        backoff_multiplier: int = 2,
        max_delay: int = 30,
        jitter: bool = False,
    ) -> Callable:
        """
        Wrap a connection plugin method with retry behavior and connection restart.

        Args:
            retries: Number of retry attempts. If None, reads from self.reconnection_retries
            initial_delay: Initial delay in seconds (default: 1)
            backoff_multiplier: Multiplier for exponential backoff (default: 2)
            max_delay: Maximum delay between retries in seconds (default: 30)
            jitter: If True, add randomness to delay values (default: False)

        Returns:
            Callable: Decorated function with retry behavior
        """

        def retry_decorator(func: Callable) -> Callable:
            @wraps(func)
            def wrapped(self, *args: Any, **kwargs: Any) -> Any:
                max_retries = cls._get_max_retries(self, retries)
                display = cls._get_display_func(self)
                cmd_summary = cls._get_cmd_summary(args)

                # Create backoff iterator for delay calculations
                backoff_iterator = BackoffIterator(
                    delay=initial_delay, backoff=backoff_multiplier, max_delay=max_delay, jitter=jitter
                )

                attempt = 0
                for pause in backoff_iterator:
                    try:
                        return_tuple = func(self, *args, **kwargs)
                        # Success - return immediately to avoid catching display() exceptions
                        return return_tuple
                    except Exception as e:
                        attempt += 1
                        if attempt > max_retries:
                            raise

                        msg = cls._format_retry_message(attempt, max_retries, e, cmd_summary, pause)
                        display(2, msg)
                        time.sleep(pause)
                        cls._restart_connection(self)

            return wrapped

        return retry_decorator



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Sabari Jaganathan (@sajagana)
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: aci_fabric_node_decommission
version_added: "2.13.0"
short_description: Manage the Commissioning and Decommissioning of the Fabric Node (fabric:RsDecommissionNode)
description:
- Manage the Commissioning and Decommissioning of the Fabric Node on Cisco ACI fabrics.
options:
  pod_id:
    description:
    - The Pod ID for the Fabric Node.
    type: int
  node_id:
    description:
    - The Node ID for the Fabric Node.
    type: int
  remove_from_controller:
    description:
    - Completely deletes the node configuration from the controller during decommissioning.
    - To register the fabric node, use M(cisco.aci.aci_fabric_node).
    - The APIC defaults to O(remove_from_controller=false) when unset during creation.
    type: bool
  state:
    description:
    - Use C(present) for decommissioning the Fabric Node.
    - Use C(absent) for commissioning the Fabric Node.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

seealso:
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fabric:RsDecommissionNode).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Sabari Jaganathan (@sajagana)
"""

EXAMPLES = r"""
- name: Decommission the fabric node
  cisco.aci.aci_fabric_node_decommission:
    host: apic
    username: admin
    password: SomeSecretPassword
    pod_id: 1
    node_id: 201
    state: present

- name: Query the decommissioned fabric node
  cisco.aci.aci_fabric_node_decommission:
    host: apic
    username: admin
    password: SomeSecretPassword
    pod_id: 1
    node_id: 201
    state: query
  register: query_one

- name: Query all decommissioned fabric nodes
  cisco.aci.aci_fabric_node_decommission:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  register: query_all

- name: Commission the fabric node
  cisco.aci.aci_fabric_node_decommission:
    host: apic
    username: admin
    password: SomeSecretPassword
    pod_id: 1
    node_id: 201
    state: absent
"""

RETURN = r"""
   current:
     description: The existing configuration from the APIC after the module has finished
     returned: success
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production environment",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   error:
     description: The error information as returned from the APIC
     returned: failure
     type: dict
     sample:
       {
           "code": "122",
           "text": "unknown managed object class foo"
       }
   raw:
     description: The raw output returned by the APIC REST API (xml or json)
     returned: parse error
     type: str
     sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class "/></imdata>'
   sent:
     description: The actual/minimal configuration pushed to the APIC
     returned: info
     type: list
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment"
               }
           }
       }
   previous:
     description: The original configuration from the APIC before the module has started
     returned: info
     type: list
     sample:
       [
           {
               "fvTenant": {
                   "attributes": {
                       "descr": "Production",
                       "dn": "uni/tn-production",
                       "name": "production",
                       "nameAlias": "",
                       "ownerKey": "",
                       "ownerTag": ""
                   }
               }
           }
       ]
   proposed:
     description: The assembled configuration from the user-provided parameters
     returned: info
     type: dict
     sample:
       {
           "fvTenant": {
               "attributes": {
                   "descr": "Production environment",
                   "name": "production"
               }
           }
       }
   filter_string:
     description: The filter string used for the request
     returned: failure or debug
     type: str
     sample: ?rsp-prop-include=config-only
   method:
     description: The HTTP method used for the request to the APIC
     returned: failure or debug
     type: str
     sample: POST
   response:
     description: The HTTP response from the APIC
     returned: failure or debug
     type: str
     sample: OK (30 bytes)
   status:
     description: The HTTP status from the APIC
     returned: failure or debug
     type: int
     sample: 200
   url:
     description: The HTTP url used for the request to the APIC
     returned: failure or debug
     type: str
     sample: https://10.11.12.13/api/mo/uni/tn-production.json
   """

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        pod_id=dict(type="int"),
        node_id=dict(type="int"),
        remove_from_controller=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["pod_id", "node_id"]],
            ["state", "absent", ["pod_id", "node_id"]],
        ],
    )

    pod_id = module.params.get("pod_id")
    node_id = module.params.get("node_id")
    remove_from_controller = module.params.get("remove_from_controller")
    state = module.params.get("state")

    aci = ACIModule(module)

    target_dn = None
    if pod_id and node_id:
        target_dn = "topology/pod-{0}/node-{1}".format(pod_id, node_id)

    aci.construct_url(
        root_class=dict(
            aci_class="fabricOOServicePol",
            aci_rn="fabric/outofsvc",
        ),
        subclass_1=dict(
            aci_class="fabricRsDecommissionNode",
            aci_rn="rsdecommissionNode-[{0}]".format(target_dn) if target_dn else None,
            target_filter={"tDn": target_dn},
        ),
    )

    aci.get_existing()

    if state == "present":  # Decommission
        aci.payload(
            aci_class="fabricRsDecommissionNode",
            class_config=dict(
                tDn=target_dn,
                removeFromController="yes" if remove_from_controller else "no",
            ),
        )
        aci.get_diff(aci_class="fabricRsDecommissionNode")
        aci.post_config()

    elif state == "absent" and aci.existing:  # Commission
        # The aci.delete_config function removes the object directly from APIC, which can interrupt the commission or decommission process before it finishes.
        # Because of that the Fabric Node may enter a bad state.
        aci.payload(
            aci_class="fabricRsDecommissionNode",
            class_config=dict(
                tDn=target_dn,
                status="deleted",
            ),
        )
        aci.get_diff(aci_class="fabricRsDecommissionNode")
        aci.post_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2026 Bill Peck, <bpeck@redhat.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_databricksworkspace
version_added: "3.20.0"
short_description: Manage Azure Databricks workspaces
description:
    - Create, update or delete Azure Databricks workspaces.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    name:
        description:
            - Name of the workspace.
        required: true
        type: str
    state:
        description:
            - Assert the state of the workspace. Use C(present) to create or update and C(absent) to delete.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - Resource location.
        type: str
    sku:
        description:
            - The SKU of the workspace.
        type: dict
        suboptions:
            name:
                description:
                    - The SKU name.
                type: str
                required: true
                choices:
                    - standard
                    - premium
                    - trial
    managed_resource_group_id:
        description:
            - The managed resource group ID.
        type: str
    parameters:
        description:
            - Custom parameters for the workspace.
        type: dict
        suboptions:
            enable_no_public_ip:
                description:
                    - Whether to enable no public IP.
                type: dict
                suboptions:
                    value:
                        description:
                            - The value to enable or disable no public IP.
                        type: bool
                        required: true
            prepare_encryption:
                description:
                    - Whether to prepare encryption.
                type: dict
                suboptions:
                    value:
                        description:
                            - The value to enable or disable prepare encryption.
                        type: bool
                        required: true
            require_infrastructure_encryption:
                description:
                    - Whether to require infrastructure encryption.
                type: dict
                suboptions:
                    value:
                        description:
                            - The value to enable or disable require infrastructure encryption.
                        type: bool
                        required: true
            custom_virtual_network_id:
                description:
                    - Custom virtual network ID.
                type: dict
                suboptions:
                    value:
                        description:
                            - The resource ID of the custom VNet.
                        type: str
                        required: true
            custom_public_subnet_name:
                description:
                    - Custom public subnet name.
                type: dict
                suboptions:
                    value:
                        description:
                            - The name of the public subnet.
                        type: str
                        required: true
            custom_private_subnet_name:
                description:
                    - Custom private subnet name.
                type: dict
                suboptions:
                    value:
                        description:
                            - The name of the private subnet.
                        type: str
                        required: true
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create a Databricks workspace
  azure_rm_databricksworkspace:
    resource_group: myResourceGroup
    name: myDatabricksWorkspace
    location: eastus
    managed_resource_group_id: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx/resourceGroups/myManagedResourceGroup"
    sku:
      name: premium
    tags:
      environment: test

- name: Create a Databricks workspace with custom VNet
  azure_rm_databricksworkspace:
    resource_group: myResourceGroup
    name: myDatabricksWorkspace
    location: eastus
    managed_resource_group_id: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxx/resourceGroups/myManagedResourceGroup"
    sku:
      name: premium
    parameters:
      custom_virtual_network_id:
        value: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myVnetRG/providers/Microsoft.Network/virtualNetworks/myVnet
      custom_public_subnet_name:
        value: public-subnet
      custom_private_subnet_name:
        value: private-subnet
      enable_no_public_ip:
        value: true

- name: Delete a Databricks workspace
  azure_rm_databricksworkspace:
    resource_group: myResourceGroup
    name: myDatabricksWorkspace
    state: absent
'''

RETURN = '''
id:
    description:
        - Workspace resource path.
    type: str
    returned: success
    example: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Databricks/workspaces/myDatabricksWorkspace"
name:
    description:
        - The name of the workspace.
    type: str
    returned: success
    example: myDatabricksWorkspace
location:
    description:
        - Resource location.
    type: str
    returned: success
    example: eastus
type:
    description:
        - Resource type.
    type: str
    returned: success
    example: Microsoft.Databricks/workspaces
sku:
    description:
        - The SKU of the workspace.
    type: dict
    returned: success
    example: {'name': 'premium'}
managed_resource_group_id:
    description:
        - The managed resource group ID.
    type: str
    returned: success
    example: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/databricks-rg-myDatabricksWorkspace-abc123
workspace_id:
    description:
        - The unique identifier of the Databricks workspace.
    type: str
    returned: success
    example: 1234567890123456
workspace_url:
    description:
        - The workspace URL.
    type: str
    returned: success
    example: adb-1234567890123456.12.azuredatabricks.net
provisioning_state:
    description:
        - The workspace provisioning state.
    type: str
    returned: success
    example: Succeeded
tags:
    description:
        - Resource tags.
    type: dict
    returned: success
    example: {'environment': 'test'}
'''  # NOQA

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    pass


class AzureRMDatabricksWorkspace(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            location=dict(type='str'),
            sku=dict(
                type='dict',
                options=dict(
                    name=dict(type='str', required=True, choices=['standard', 'premium', 'trial']),
                )
            ),
            managed_resource_group_id=dict(type='str'),
            parameters=dict(
                type='dict',
                options=dict(
                    enable_no_public_ip=dict(
                        type='dict',
                        options=dict(
                            value=dict(type='bool', required=True)
                        )
                    ),
                    prepare_encryption=dict(
                        type='dict',
                        options=dict(
                            value=dict(type='bool', required=True)
                        )
                    ),
                    require_infrastructure_encryption=dict(
                        type='dict',
                        options=dict(
                            value=dict(type='bool', required=True)
                        )
                    ),
                    custom_virtual_network_id=dict(
                        type='dict',
                        options=dict(
                            value=dict(type='str', required=True)
                        )
                    ),
                    custom_public_subnet_name=dict(
                        type='dict',
                        options=dict(
                            value=dict(type='str', required=True)
                        )
                    ),
                    custom_private_subnet_name=dict(
                        type='dict',
                        options=dict(
                            value=dict(type='str', required=True)
                        )
                    )
                )
            )
        )

        self.results = dict(
            changed=False,
            id=None
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.location = None
        self.sku = None
        self.managed_resource_group_id = None
        self.parameters = None

        required_if = [
            ('state', 'present', ['managed_resource_group_id']),
        ]

        super(AzureRMDatabricksWorkspace, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True, required_if=required_if)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        self.results = dict(
            compare=[],
        )

        changed = False

        if not self.location:
            resource_group = self.get_resource_group(self.resource_group)
            self.location = resource_group.location

        workspace = self.get_workspace()

        if not workspace and self.state == 'present':
            workspace_obj = self.create_workspace_obj()
            changed = True
            if not self.check_mode:
                workspace = self.create_or_update_workspace(workspace_obj)
        elif workspace and self.state == 'present':
            update_tags, new_tags = self.update_tags(workspace.tags)
            if update_tags:
                self.tags = new_tags

            workspace_obj = self.create_workspace_obj()

            changed = not self.default_compare({},
                                               workspace_obj.as_dict(),
                                               workspace.as_dict(),
                                               '',
                                               self.results)

            if changed and not self.check_mode:
                workspace = self.create_or_update_workspace(workspace_obj)
        elif workspace and self.state == 'absent':
            changed = True
            workspace = None
            if not self.check_mode:
                self.delete_workspace()

        if workspace:
            self.results = workspace.as_dict()

        self.results['changed'] = changed
        return self.results

    def create_workspace_obj(self):
        try:
            workspace_params = {
                'location': self.location,
                'tags': self.tags
            }

            if self.sku:
                workspace_params['sku'] = self.databricks_models.Sku(
                    name=self.sku.get('name'),
                )

            if self.managed_resource_group_id:
                workspace_params['managed_resource_group_id'] = self.managed_resource_group_id

            if self.parameters:
                parameters = {}

                if 'enable_no_public_ip' in self.parameters and self.parameters['enable_no_public_ip']:
                    parameters['enable_no_public_ip'] = self.databricks_models.WorkspaceCustomBooleanParameter(
                        value=self.parameters['enable_no_public_ip']['value']
                    )

                if 'prepare_encryption' in self.parameters and self.parameters['prepare_encryption']:
                    parameters['prepare_encryption'] = self.databricks_models.WorkspaceCustomBooleanParameter(
                        value=self.parameters['prepare_encryption']['value']
                    )

                if 'require_infrastructure_encryption' in self.parameters and self.parameters['require_infrastructure_encryption']:
                    parameters['require_infrastructure_encryption'] = self.databricks_models.WorkspaceCustomBooleanParameter(
                        value=self.parameters['require_infrastructure_encryption']['value']
                    )

                if 'custom_virtual_network_id' in self.parameters and self.parameters['custom_virtual_network_id']:
                    parameters['custom_virtual_network_id'] = self.databricks_models.WorkspaceCustomStringParameter(
                        value=self.parameters['custom_virtual_network_id']['value']
                    )

                if 'custom_public_subnet_name' in self.parameters and self.parameters['custom_public_subnet_name']:
                    parameters['custom_public_subnet_name'] = self.databricks_models.WorkspaceCustomStringParameter(
                        value=self.parameters['custom_public_subnet_name']['value']
                    )

                if 'custom_private_subnet_name' in self.parameters and self.parameters['custom_private_subnet_name']:
                    parameters['custom_private_subnet_name'] = self.databricks_models.WorkspaceCustomStringParameter(
                        value=self.parameters['custom_private_subnet_name']['value']
                    )
                workspace_params['parameters'] = self.databricks_models.WorkspaceCustomParameters(**parameters)

            return self.databricks_models.Workspace(**workspace_params)
        except Exception as exc:
            self.fail('Error creating workspace_obj {0} - {1}'.format(self.name, str(exc)))

    def create_or_update_workspace(self, workspace_obj):
        try:
            poller = self.databricks_client.workspaces.begin_create_or_update(
                self.resource_group,
                self.name,
                workspace_obj
            )
            return self.get_poller_result(poller)
        except Exception as exc:
            self.fail('Error creating/updating workspace {0} - {1}'.format(self.name, str(exc)))

    def get_workspace(self):
        try:
            return self.databricks_client.workspaces.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass
        return None

    def delete_workspace(self):
        try:
            poller = self.databricks_client.workspaces.begin_delete(self.resource_group, self.name)
            return self.get_poller_result(poller)
        except Exception as exc:
            self.fail('Error deleting workspace {0} - {1}'.format(self.name, str(exc)))


def main():
    AzureRMDatabricksWorkspace()


if __name__ == '__main__':
    main()



# Copyright (c) 2014, Red Hat, Inc.
# Copyright (c) 2014, Tim Bielawa <tbielawa@redhat.com>
# Copyright (c) 2014, Magnus Hedemark <mhedemar@redhat.com>
# Copyright (c) 2017, Dag Wieers <dag@wieers.com>
# Copyright (c) 2026, Shreyash Bhosale <shreyashpb16@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import json
import os
import typing as t
from io import BytesIO

from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.general.plugins.module_utils import _deps as deps
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule

with deps.declare("lxml"):
    from lxml import etree


def get_common_argument_spec(*, xpath_required: bool = False) -> dict[str, dict[str, t.Any]]:
    """Return the argument spec shared by xml and xml_info modules."""
    return dict(
        path=dict(type="path", aliases=["dest", "file"]),
        xmlstring=dict(type="str"),
        xpath=dict(type="str", required=xpath_required),
        namespaces=dict(type="dict", default={}),
        strip_cdata_tags=dict(type="bool", default=False),
        huge_tree=dict(type="bool", default=False),
    )


def check_lxml(module: AnsibleModule) -> None:
    deps.validate(module, "lxml")
    version_str = ".".join(str(f) for f in etree.LXML_VERSION)
    if LooseVersion(version_str) < LooseVersion("2.3.0"):
        module.fail_json(msg="The xml ansible module requires lxml 2.3.0 or newer installed on the managed machine")
    elif LooseVersion(version_str) < LooseVersion("3.0.0"):
        module.warn("Using lxml version lower than 3.0.0 does not guarantee predictable element attribute order.")


def validate_xpath(module: AnsibleModule, xpath: str) -> None:
    try:
        etree.XPath(xpath)
    except etree.XPathSyntaxError as e:
        module.fail_json(msg=f"Syntax error in xpath expression: {xpath} ({e})")
    except etree.XPathEvalError as e:
        module.fail_json(msg=f"Evaluation error in xpath expression: {xpath} ({e})")


def parse_xml_doc(
    module: AnsibleModule,
    xml_file: str | None = None,
    xml_string: str | None = None,
    strip_cdata_tags: bool = False,
    huge_tree: bool = False,
    remove_blank_text: bool = False,
    resolve_entities: bool = True,
) -> etree._ElementTree:
    infile: t.IO[bytes] | None = None
    try:
        if xml_string:
            infile = BytesIO(to_bytes(xml_string, errors="surrogate_or_strict"))
        elif xml_file:
            if not os.path.isfile(xml_file):
                module.fail_json(msg=f"The target XML source '{xml_file}' does not exist.")
            if not os.access(xml_file, os.R_OK):
                module.fail_json(msg=f"The target XML source '{xml_file}' is not readable.")
            infile = open(xml_file, "rb")  # noqa: SIM115
        else:
            module.fail_json(msg="No XML source provided; specify 'path' or 'xmlstring'.")

        parser = etree.XMLParser(
            remove_blank_text=remove_blank_text,
            strip_cdata=strip_cdata_tags,
            huge_tree=huge_tree,
            resolve_entities=resolve_entities,
        )
        doc = etree.parse(infile, parser)
    except etree.XMLSyntaxError as e:
        module.fail_json(msg=f"Error while parsing document: {xml_file or 'xml_string'} ({e})")
    except OSError as e:
        module.fail_json(msg=f"Error reading XML source: {xml_file or 'xml_string'} ({e})")
    finally:
        if infile:
            infile.close()

    return doc


def xpath_matches(tree: t.Any, xpath: str, namespaces: dict[str, str]) -> bool:
    """Test if a node exists."""
    return bool(tree.xpath(xpath, namespaces=namespaces))


def is_node(tree: t.Any, xpath: str, namespaces: dict[str, str]) -> bool:
    """Test if a given xpath matches anything and if that match is a node."""
    match = tree.xpath(xpath, namespaces=namespaces)
    return bool(match) and isinstance(match[0], etree._Element)


def get_matches(tree: t.Any, xpath: str, namespaces: dict[str, str]) -> tuple[list[str], str]:
    """Return matched XPath paths as (match_xpaths, message)."""
    match = tree.xpath(xpath, namespaces=namespaces)
    match_xpaths: list[str] = [tree.getpath(m) for m in match]
    match_str = json.dumps(match_xpaths)
    msg = f"selector '{xpath}' match: {match_str}"
    return match_xpaths, msg


def count_matches(tree: t.Any, xpath: str, namespaces: dict[str, str], *, count_mode: str = "match") -> tuple[int, str]:
    """Return the count of nodes matching the xpath as (count, message).

    When count_mode is "xpath", uses the XPath count() function for better memory efficiency.
    When count_mode is "match" (default), evaluates the expression and counts with len().
    """
    if count_mode == "xpath":
        hits = int(tree.xpath(f"count({xpath})", namespaces=namespaces))
    else:
        hits = len(tree.xpath(xpath, namespaces=namespaces))
    msg = f"found {hits} nodes"
    return hits, msg


def collect_element_text(tree: t.Any, xpath: str, namespaces: dict[str, str]) -> list[tuple[str, str | None]] | None:
    """Get text content of matched elements as (tag, text) tuples. Returns None if xpath does not match a node."""
    match = tree.xpath(xpath, namespaces=namespaces)
    if not match or not isinstance(match[0], etree._Element):
        return None
    return [(element.tag, element.text) for element in match]


def collect_element_attr(
    tree: t.Any, xpath: str, namespaces: dict[str, str]
) -> list[tuple[str, dict[str, str]]] | None:
    """Get attributes of matched elements as (tag, attributes) tuples. Returns None if xpath does not match a node."""
    match = tree.xpath(xpath, namespaces=namespaces)
    if not match or not isinstance(match[0], etree._Element):
        return None
    return [(element.tag, dict(element.attrib)) for element in match]



#!/usr/bin/python
#
# Copyright (c) 2018 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_servicebus
version_added: "0.1.2"
short_description: Manage Azure Service Bus
description:
    - Create, update or delete an Azure Service Bus namespaces.
options:
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    name:
        description:
            - Name of the servicebus namespace.
        required: true
        type: str
    state:
        description:
            - Assert the state of the servicebus. Use C(present) to create or update and use C(absen) to delete.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - The servicebus's location.
        type: str
    sku:
        description:
            - Namespace SKU.
        type: str
        choices:
            - standard
            - basic
            - premium
        default: standard
    minimum_tls_version:
        description:
            - The minimum TLS version for the cluster to support.
        type: str
        choices:
            - '1.0'
            - '1.1'
            - '1.2'
    zone_redundant:
        description:
            - Enabling this property creates a Premium Service Bus Namespace in regions supported availability zones.
        type: bool
    disable_local_auth:
        description:
            - This property disables SAS authentication for the Service Bus namespace.
        type: bool
    public_network_access:
        description:
            - This determines if traffic is allowed over public network.
            - By default it is C(Enabled).
        type: str
        default: Enabled
        choices:
            - Enabled
            - Disabled
            - SecuredByPerimeter
    premium_messaging_partitions:
        description:
            - The number of partitions of a Service Bus namespace.
            - This property is only applicable to C(premium) SKU namespaces.
            - It default value is C(1) for C(premium) namespaces.
        type: int
        choices:
            - 1
            - 2
            - 4

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
    - azure.azcollection.azure_identity_multiple

author:
    - Yuwei Zhou (@yuwzho)

'''

EXAMPLES = '''
- name: Create a namespace
  azure_rm_servicebus:
    name: deadbeef
    location: eastus
    tags:
      key1: value1
'''
RETURN = '''
id:
    description:
        - Current state of the service bus.
    returned: success
    type: str
    sample: "/subscriptions/xxx...xxx/resourceGroups/myResourceGroup/providers/Microsoft.ServiceBus/namespaces/myServicebus"
'''

try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
    from azure.mgmt.servicebus.v2022_10_01_preview.models import (Identity, UserAssignedIdentity)
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible.module_utils._text import to_native
from datetime import datetime, timedelta


class AzureRMServiceBus(AzureRMModuleBaseExt):

    def __init__(self):

        self.module_arg_spec = dict(
            resource_group=dict(type='str', required=True),
            name=dict(type='str', required=True),
            location=dict(type='str'),
            state=dict(type='str', default='present', choices=['present', 'absent']),
            sku=dict(type='str', choices=['basic', 'standard', 'premium'], default='standard'),
            identity=dict(
                type="dict",
                options=self.managed_identity_multiple_spec
            ),
            minimum_tls_version=dict(type='str', choices=['1.0', '1.1', '1.2']),
            zone_redundant=dict(type='bool'),
            disable_local_auth=dict(type='bool'),
            public_network_access=dict(type='str', default='Enabled', choices=["Enabled", "Disabled", "SecuredByPerimeter"]),
            premium_messaging_partitions=dict(type='int', choices=[1, 2, 4])
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.sku = None
        self.location = None
        self._managed_identity = None
        self.identity = None
        self.update_identity = False
        self.minimum_tls_version = None
        self.zone_redundant = None
        self.disable_local_auth = None
        self.public_network_access = None
        self.premium_messaging_partitions = None

        self.results = dict(
            changed=False,
            id=None
        )

        super(AzureRMServiceBus, self).__init__(self.module_arg_spec,
                                                supports_tags=True,
                                                supports_check_mode=True)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {
                "identity": Identity,
                "user_assigned": UserAssignedIdentity,
            }
        return self._managed_identity

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        if not self.location:
            resource_group = self.get_resource_group(self.resource_group)
            self.location = resource_group.location

        original = self.get()

        if not original:
            self.check_name()

        curr_identity = original.identity.as_dict() if original and original.identity else None

        if self.identity:
            self.update_identity, identity_result = self.update_managed_identity(curr_identity=curr_identity,
                                                                                 new_identity=self.identity)
            self.identity = identity_result.as_dict()

        if self.state == 'present':
            if self.sku != 'premium' and self.premium_messaging_partitions is not None:
                self.fail("The premium_messaging_partitions is only applicable to C(premium) SKU namespaces.")

            if not self.check_mode:
                if original:
                    update_tags, new_tags = self.update_tags(original.tags)
                    if update_tags:
                        changed = True
                        self.tags = new_tags
                    if self.update_identity:
                        changed = True
                    if self.minimum_tls_version is not None and self.minimum_tls_version != original.minimum_tls_version:
                        changed = True
                    else:
                        self.minimum_tls_version = original.minimum_tls_version
                    if self.premium_messaging_partitions is not None and self.premium_messaging_partitions != original.premium_messaging_partitions:
                        changed = True
                    else:
                        self.premium_messaging_partitions = original.premium_messaging_partitions
                    if self.public_network_access is not None and self.public_network_access != original.public_network_access:
                        changed = True
                    else:
                        self.public_network_access = original.public_network_access
                    if self.zone_redundant is not None and bool(self.zone_redundant) != bool(original.zone_redundant):
                        # changed = True
                        # self.fail("The zone_redundant is an immutable property")
                        self.log("The default value of zone_redundant is True and cannot be set")
                    else:
                        self.zone_redundant = original.zone_redundant
                    if self.disable_local_auth is not None and bool(self.disable_local_auth) != bool(original.disable_local_auth):
                        changed = True
                    else:
                        self.disable_local_auth = original.disable_local_auth
                    if changed:
                        original = self.create()
                else:
                    changed = True
                    original = self.create()
            else:
                changed = True
        elif self.state == 'absent' and original:
            changed = True
            original = None
            if not self.check_mode:
                self.delete()
                self.results['deleted'] = True

        if original:
            self.results = self.to_dict(original)
        self.results['changed'] = changed
        return self.results

    def check_name(self):
        try:
            check_name = self.servicebus_client.namespaces.check_name_availability(parameters={'name': self.name})
            if not check_name or not check_name.name_available:
                self.fail("Error creating namespace {0} - {1}".format(self.name, check_name.message or str(check_name)))
        except Exception as exc:
            self.fail("Error creating namespace {0} - {1}".format(self.name, exc.message or str(exc)))

    def create(self):
        self.log('Cannot find namespace, creating a one')
        try:
            sku = self.servicebus_models.SBSku(name=str.capitalize(self.sku))
            parameters = self.servicebus_models.SBNamespace(location=self.location,
                                                            tags=self.tags,
                                                            sku=sku,
                                                            minimum_tls_version=self.minimum_tls_version,
                                                            zone_redundant=self.zone_redundant,
                                                            disable_local_auth=self.disable_local_auth,
                                                            public_network_access=self.public_network_access,
                                                            premium_messaging_partitions=self.premium_messaging_partitions,
                                                            identity=self.identity)
            poller = self.servicebus_client.namespaces.begin_create_or_update(self.resource_group, self.name, parameters)
            ns = self.get_poller_result(poller)
        except Exception as exc:
            self.fail('Error creating namespace {0} - {1}'.format(self.name, str(exc)))
        return ns

    def delete(self):
        try:
            self.servicebus_client.namespaces.begin_delete(self.resource_group, self.name)
            return True
        except Exception as exc:
            self.fail("Error deleting route {0} - {1}".format(self.name, str(exc)))

    def get(self):
        try:
            return self.servicebus_client.namespaces.get(self.resource_group, self.name)
        except Exception:
            return None

    def to_dict(self, instance):
        result = dict()
        attribute_map = self.servicebus_models.SBNamespace._attribute_map
        for attribute in attribute_map.keys():
            value = getattr(instance, attribute)
            if not value:
                continue
            if isinstance(value, self.servicebus_models.SBSku):
                result[attribute] = value.name.lower()
            elif isinstance(value, datetime):
                result[attribute] = str(value)
            elif isinstance(value, str):
                result[attribute] = to_native(value)
            elif attribute == 'max_size_in_megabytes':
                result['max_size_in_mb'] = value
            elif isinstance(value, Identity):
                result['identity'] = value.as_dict()
            else:
                result[attribute] = value
        return result


def is_valid_timedelta(value):
    if value == timedelta(10675199, 10085, 477581):
        return None
    return value


def main():
    AzureRMServiceBus()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


"""
The index_of filter plugin
"""
from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    name: index_of
    author: Bradley Thornton (@cidrblock)
    version_added: "1.0.0"
    short_description: Find the indices of items in a list matching some criteria
    description:
        - This plugin returns the indices of items matching some criteria in a list.
        - When working with a list of dictionaries, the key to evaluate can be specified.
        - B(index_of) is also available as a B(lookup plugin) for convenience.
        - Using the parameters below- C(data|ansible.utils.index_of(test, value, key, fail_on_missing, wantlist))
    options:
      data:
        description:
        - A list of items to enumerate and test against.
        - This option represents the value that is passed to the filter plugin in pipe format.
        - For example C(config_data|ansible.utils.index_of('x')), in this case C(config_data) represents this option.
        type: list
        required: True
      test:
        description:
        - The name of the test to run against the list, a valid jinja2 test or ansible test plugin.
        - Jinja2 includes the following tests U(http://jinja.palletsprojects.com/templates/#builtin-tests).
        - An overview of tests included in ansible U(https://docs.ansible.com/ansible/latest/user_guide/playbooks_tests.html)
        type: str
        required: True
      value:
        description:
        - The value used to test each list item against.
        - 'Not required for simple tests (eg: C(true), C(false), C(even), C(odd))'
        - May be a C(string), C(boolean), C(number), C(regular expression) C(dict) and so on, depending on the C(test) used
        type: raw
      key:
        description:
        - When the data provided is a list of dictionaries, run the test against this dictionary key.
        - When using a I(key), the I(data) must only contain dictionaries.
        - See I(fail_on_missing) below to determine the behavior when the I(key) is missing from a dictionary in the I(data).
        type: str
      fail_on_missing:
        description: When provided a list of dictionaries, fail if the key is missing from one or more of the dictionaries.
        type: bool
      wantlist:
        description:
        - When only a single entry in the I(data) is matched, the index of that entry is returned as an integer.
        - If set to C(True), the return value will always be a list, even if only a single entry is matched.
        type: bool

    notes:
"""

EXAMPLES = r"""

#### Simple examples

- name: Define a list
  ansible.builtin.set_fact:
    data:
      - 1
      - 2
      - 3

- name: Find the index of 2
  ansible.builtin.set_fact:
    indices: "{{ data|ansible.utils.index_of('eq', 2) }}"

# TASK [Find the index of 2] *************************************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     indices: '1'


- name: Find the index of 2, ensure list is returned
  ansible.builtin.set_fact:
    indices: "{{ data|ansible.utils.index_of('eq', 2, wantlist=True) }}"

# TASK [Find the index of 2, ensure list is returned] ************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     indices:
#     - 1


- name: Find the index of 3 using the long format
  ansible.builtin.set_fact:
    indices: "{{ data|ansible.utils.index_of(test='eq', value=value, wantlist=True) }}"
  vars:
    value: 3

# TASK [Find the index of 3 using the long format] ***************************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     indices:
#     - 2


- name: Find numbers greater than 1, using loop
  debug:
    msg: "{{ data[item] }} is {{ test }} than {{ value }}"
  loop: "{{ data|ansible.utils.index_of(test, value) }}"
  vars:
    test: '>'
    value: 1

# TASK [Find numbers great than 1, using loop] *******************************
# ok: [sw01] => (item=1) =>
#   msg: 2 is > than 1
# ok: [sw01] => (item=2) =>
#   msg: 3 is > than 1


#### Working with lists of dictionaries

- name: Define a list with hostname and type
  ansible.builtin.set_fact:
    data:
      - name: sw01.example.lan
        type: switch
      - name: rtr01.example.lan
        type: router
      - name: fw01.example.corp
        type: firewall
      - name: fw02.example.corp
        type: firewall

- name: Find the index of all firewalls using the type key
  ansible.builtin.set_fact:
    firewalls: "{{ data|ansible.utils.index_of('eq', 'firewall', 'type') }}"

# TASK [Find the index of all firewalls using the type key] ******************
# ok: [nxos101] => changed=false
#   ansible_facts:
#     firewalls:
#     - 2
#     - 3

- name: Find the index of all firewalls, use in a loop
  debug:
    msg: "The type of {{ device_type }} at index {{ item }} has name {{ data[item].name }}."
  loop: "{{ data|ansible.utils.index_of('eq', device_type, 'type') }}"
  vars:
    device_type: firewall

# TASK [Find the index of all firewalls, use in a loop, as a filter] *********
# ok: [nxos101] => (item=2) =>
#   msg: The type of firewall at index 2 has name fw01.example.corp.
# ok: [nxos101] => (item=3) =>
#   msg: The type of firewall at index 3 has name fw02.example.corp.

- name: Find the index of all devices with a .corp name
  debug:
    msg: "The device named {{ data[item].name }} is a {{ data[item].type }}"
  loop: "{{ data|ansible.utils.index_of('regex', expression, 'name') }}"
  vars:
    expression: '\.corp$'

# TASK [Find the index of all devices with a .corp name] *********************
# ok: [nxos101] => (item=2) =>
#   msg: The device named fw01.example.corp is a firewall
# ok: [nxos101] => (item=3) =>
#   msg: The device named fw02.example.corp is a firewall


#### Working with complex structures from resource modules

- name: Retrieve the current L3 interface configuration
  cisco.nxos.nxos_l3_interfaces:
    state: gathered
  register: current_l3

# TASK [Retrieve the current L3 interface configuration] *********************
# ok: [sw01] => changed=false
#   gathered:
#   - name: Ethernet1/1
#   - name: Ethernet1/2
#   <...>
#   - name: Ethernet1/128
#   - ipv4:
#     - address: 192.168.101.14/24
#     name: mgmt0

- name: Find the indices interfaces with a 192.168.101.xx ip address
  ansible.builtin.set_fact:
    found: "{{ found + entry }}"
  with_indexed_items: "{{ current_l3.gathered }}"
  vars:
    found: []
    ip: '192.168.101.'
    address: "{{ item.1.ipv4|d([])|ansible.utils.index_of('search', ip, 'address', wantlist=True) }}"
    entry:
      - interface_idx: "{{ item.0 }}"
        address_idxs: "{{ address }}"
  when: address

# TASK [debug] ***************************************************************
# ok: [sw01] =>
#   found:
#   - address_idxs:
#     - 0
#     interface_idx: '128'

- name: Show all interfaces and their address
  debug:
    msg: "{{ interface.name }} has ip {{ address }}"
  loop: "{{ found|subelements('address_idxs') }}"
  vars:
    interface: "{{ current_l3.gathered[item.0.interface_idx|int] }}"
    address: "{{ interface.ipv4[item.1].address }}"

# TASK [Show all interfaces and their address] *******************************
# ok: [nxos101] => (item=[{'interface_idx': '128', 'address_idxs': [0]}, 0]) =>
#   msg: mgmt0 has ip 192.168.101.14/24


#### Working with deeply nested data

- name: Define interface configuration facts
  ansible.builtin.set_fact:
    data:
      interfaces:
        interface:
          - config:
              description: configured by Ansible - 1
              enabled: true
              loopback-mode: false
              mtu: 1024
              name: loopback0000
              type: eth
            name: loopback0000
            subinterfaces:
              subinterface:
                - config:
                    description: subinterface configured by Ansible - 1
                    enabled: true
                    index: 5
                  index: 5
                - config:
                    description: subinterface configured by Ansible - 2
                    enabled: false
                    index: 2
                  index: 2
          - config:
              description: configured by Ansible - 2
              enabled: false
              loopback-mode: false
              mtu: 2048
              name: loopback1111
              type: virt
            name: loopback1111
            subinterfaces:
              subinterface:
                - config:
                    description: subinterface configured by Ansible - 3
                    enabled: true
                    index: 10
                  index: 10
                - config:
                    description: subinterface configured by Ansible - 4
                    enabled: false
                    index: 3
                  index: 3


- name: Find the description of loopback111, subinterface index 10
  debug:
    msg: |-
      {{ data.interfaces.interface[int_idx|int]
          .subinterfaces.subinterface[subint_idx|int]
            .config.description }}
  vars:
    # the values to search for
    int_name: loopback1111
    sub_index: 10
    # retrieve the index in each nested list
    int_idx: |
      {{ data.interfaces.interface|
            ansible.utils.index_of('eq', int_name, 'name') }}
    subint_idx: |
      {{ data.interfaces.interface[int_idx|int]
            .subinterfaces.subinterface|
                ansible.utils.index_of('eq', sub_index, 'index') }}

# TASK [Find the description of loopback111, subinterface index 10] ************
# ok: [sw01] =>
#   msg: subinterface configured by Ansible - 3
"""

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.index_of import index_of


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment


@pass_environment
def _index_of(*args, **kwargs):
    """Find the indicies of items in a list matching some criteria."""

    keys = [
        "environment",
        "data",
        "test",
        "value",
        "key",
        "fail_on_missing",
        "wantlist",
    ]
    data = dict(zip(keys, args))
    data.update(kwargs)
    environment = data.pop("environment")
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="index_of")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    updated_data["tests"] = environment.tests
    return index_of(**updated_data)


class FilterModule(object):
    """index_of"""

    def filters(self):
        """a mapping of filter names to functions"""
        return {"index_of": _index_of}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_snmp_policy_info
short_description: Gather information about SNMP Policies in Cisco Intersight
description:
  - Gather information about SNMP Policies in L(Cisco Intersight,https://intersight.com).
  - Information can be filtered by O(organization) and O(name).
  - If no filters are passed, all SNMP Policies will be returned.
extends_documentation_fragment: intersight
options:
  organization:
    description:
      - The name of the organization the SNMP Policy belongs to.
    type: str
  name:
    description:
      - The name of the SNMP Policy to gather information from.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Fetch a specific SNMP Policy by name
  cisco.intersight.intersight_snmp_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "mixed-snmp-policy"

- name: Fetch all SNMP Policies in a specific Organization
  cisco.intersight.intersight_snmp_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"

- name: Fetch all SNMP Policies
  cisco.intersight.intersight_snmp_policy_info:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": [
    {
        "Name": "mixed-snmp-policy",
        "ObjectType": "snmp.Policy",
        "Enabled": true,
        "V2Enabled": true,
        "V3Enabled": true,
        "SnmpPort": 161,
        "SysContact": "admin@example.com",
        "SysLocation": "Data Center A",
        "CommunityAccess": "Full",
        "AccessCommunityString": "public",
        "TrapCommunity": "trapcomm",
        "EngineId": "custom-engine-id",
        "SnmpUsers": [
            {
                "Name": "admin",
                "SecurityLevel": "AuthPriv",
                "AuthType": "SHA",
                "IsAuthPasswordSet": true,
                "PrivacyType": "AES",
                "IsPrivacyPasswordSet": true
            }
        ],
        "SnmpTraps": [
            {
                "Enabled": true,
                "Version": "V2",
                "Community": "trapcomm",
                "Type": "Trap",
                "Destination": "192.168.1.100",
                "Port": 162
            }
        ],
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    },
    {
        "Name": "v3-only-snmp-policy",
        "ObjectType": "snmp.Policy",
        "Enabled": true,
        "V3Enabled": true,
        "SnmpPort": 161,
        "SysContact": "admin@example.com",
        "SysLocation": "Data Center B",
        "CommunityAccess": "Disabled",
        "EngineId": "v3-engine-id",
        "SnmpUsers": [
            {
                "Name": "v3user",
                "SecurityLevel": "AuthPriv",
                "AuthType": "SHA",
                "IsAuthPasswordSet": true,
                "PrivacyType": "AES",
                "IsPrivacyPasswordSet": true
            }
        ],
        "SnmpTraps": [],
        "Tags": []
    }
  ]
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        organization=dict(type='str'),
        name=dict(type='str')
    )
    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to fetch info
    resource_path = '/snmp/Policies'

    query_params = intersight.set_query_params()

    intersight.get_resource(
        resource_path=resource_path,
        query_params=query_params,
        return_list=True
    )

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Yunge Zhu, <yungez@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_keyvault_info
version_added: "0.1.2"
short_description: Get Azure Key Vault facts
description:
    - Get facts of Azure Key Vault.

options:
    resource_group:
        description:
            - The name of the resource group to which the key vault belongs.
        type: str
    name:
        description:
            - The name of the key vault.
        type: str
    hsm_name:
        description:
            - The name of the HSM.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yunge Zhu (@yungezz)

'''

EXAMPLES = '''
- name: Get Key Vault by name
  azure_rm_keyvault_info:
    resource_group: myResourceGroup
    name: myVault

- name: List Key Vaults in specific resource group
  azure_rm_keyvault_info:
    resource_group: myResourceGroup

- name: List Key Vaults in current subscription
  azure_rm_keyvault_info:
'''

RETURN = '''
keyvaults:
    description: List of Azure Key Vaults.
    returned: always
    type: list
    contains:
        name:
            description:
                - Name of the vault.
            returned: always
            type: str
            sample: myVault
        id:
            description:
                - Resource Id of the vault.
            returned: always
            type: str
            sample: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.KeyVault/vaults/myVault
        vault_uri:
            description:
                - Vault uri.
            returned: always
            type: str
            sample: https://myVault.vault.azure.net/
        location:
            description:
                - Location of the vault.
            returned: always
            type: str
            sample: eastus
        enabled_for_deployments:
            description:
                - Whether Azure Virtual Machines are permitted to retrieve certificates stored as secrets from the key vault.
            returned: always
            type: bool
            sample: False
        enabled_for_disk_encryption:
            description:
                - Whether Azure Disk Encryption is permitted to retrieve secrets from the vault and unwrap keys.
            returned: always
            type: bool
            sample: False
        enabled_for_template_deployment:
            description:
                - Whether Azure Resource Manager is permitted to retrieve secrets from the key vault.
            returned: always
            type: bool
            sample: False
        enable_rbac_authorization:
            description:
                - Property that controls how data actions are authorized.
            returned: always
            type: bool
            sample: False
        enable_soft_delete:
            description:
                - Property to specify whether Azure Resource Manager is permitted to retrieve secrets from the key vault.
            type: bool
            returned: always
            sample: True
        enable_purge_protection:
            description:
                - Property specifying whether protection against purge is enabled for this vault.
            type: bool
            returned: always
            sample: False
        soft_delete_retention_in_days:
            description:
                - Property specifying the number of days to retain deleted vaults.
            type: int
            returned: always
            sample: 90
        tags:
            description:
                - List of tags.
            type: list
            sample:
                - foo
        sku:
            description:
                - Sku of the vault.
            returned: always
            type: dict
            contains:
                family:
                    description: Sku family name.
                    type: str
                    returned: always
                    sample: A
                name:
                    description: Sku name.
                    type: str
                    returned: always
                    sample: standard
        public_network_access:
            description:
                - Property to specify whether the vault will accept traffic from public internet.
            type: str
            returned: always
            sample: Disabled
        network_acls:
            description:
                - A collection of rules governing the accessibility of the vault from specific network locations.
            returned: always
            type: complex
            contains:
                bypass:
                    description:
                        - Tells what traffic can bypass network rules.
                    type: str
                    returned: always
                    sample: AzureServices
                default_action:
                    description:
                        - The default action when no rule from ipRules and from virtualNetworkRules match.
                    type: str
                    returned: always
                    sample: Allow
                ip_rules:
                    description:
                        - The list of IP address rules.
                    type: list
                    returned: always
                    sample: [{'value': '124.56.78.91/32'}]
                virtual_network_rules:
                    description:
                        - The list of virtual network rules.
                    type: list
                    returned: always
                    sample: [{'id': "/subscriptions/**/resourcegroups/**/providers/microsoft.network/virtualnetworks/**/subnets/subnet01",
                            'ignore_missing_vnet_service_endpoint': True}]
        access_policies:
            description:
                - List of policies.
            returned: always
            type: complex
            contains:
                object_id:
                    description: The object if of a user, service principal or security group in AAD for the vault.
                    type: str
                    returned: always
                    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
                tenant_id:
                    description: The AAD tenant iD that should be used for authenticating requests to the key vault.
                    type: str
                    returned: always
                    sample: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
                permissions:
                    description: Permissions the identity has for keys, secrets and certificates.
                    type: complex
                    returned: always
                    contains:
                        key_permissions:
                            description:
                                Permissions to keys.
                            type: list
                            returned: always
                            sample:
                                - get
                                - create
                        keys:
                            description:
                                - Permissions to keys.
                                - This return value has been deprecated and will be removed in a
                                  release after 2027-05-01. Use C(key_permissions) instead.
                            type: list
                            returned: always
                            sample:
                                - get
                                - create
                        secrets:
                            description:
                                Permissions to secrets.
                            type: list
                            returned: always
                            sample:
                                - list
                                - set
                        certificates:
                            description:
                                Permissions to secrets.
                            type: list
                            returned: always
                            sample:
                                - get
                                - import
'''


from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.keyvault import KeyVaultManagementClient
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


def keyvault_to_dict(vault):
    return dict(
        id=vault.id,
        name=vault.name,
        location=vault.location,
        tags=vault.tags,
        vault_uri=vault.properties.vault_uri,
        enabled_for_deployment=vault.properties.enabled_for_deployment,
        enabled_for_disk_encryption=vault.properties.enabled_for_disk_encryption,
        enabled_for_template_deployment=vault.properties.enabled_for_template_deployment,
        enable_soft_delete=vault.properties.enable_soft_delete,
        enable_rbac_authorization=vault.properties.enable_rbac_authorization,
        soft_delete_retention_in_days=vault.properties.soft_delete_retention_in_days
        if vault.properties.soft_delete_retention_in_days else 90,
        enable_purge_protection=vault.properties.enable_purge_protection
        if vault.properties.enable_purge_protection else False,
        access_policies=[dict(
            tenant_id=policy.tenant_id,
            object_id=policy.object_id,
            permissions=dict(
                key_permissions=[kp.lower() for kp in policy.permissions.keys] if policy.permissions.keys else None,
                keys=[kp.lower() for kp in policy.permissions.keys] if policy.permissions.keys else None,
                secrets=[sp.lower() for sp in policy.permissions.secrets] if policy.permissions.secrets else None,
                certificates=[cp.lower() for cp in policy.permissions.certificates] if policy.permissions.certificates else None,
                storage=[stp.lower() for stp in policy.permissions.storage] if policy.permissions.storage else None
            ) if policy.permissions else None,
        ) for policy in vault.properties.access_policies] if vault.properties.access_policies else None,
        sku=dict(
            family=vault.properties.sku.family,
            name=vault.properties.sku.name
        ),
        public_network_access=vault.properties.public_network_access,
        network_acls=dict(
            bypass=vault.properties.network_acls.bypass,
            default_action=vault.properties.network_acls.default_action,
            ip_rules=[dict(
                value=item.value
            ) for item in vault.properties.network_acls.ip_rules] if vault.properties.network_acls.ip_rules else None,
            virtual_network_rules=[dict(
                id=item.id,
                ignore_missing_vnet_service_endpoint=item.ignore_missing_vnet_service_endpoint
            ) for item in vault.properties.network_acls.virtual_network_rules] if vault.properties.network_acls.virtual_network_rules else None
        ) if vault.properties.network_acls else None
    )


def hsm_to_dict(hsm):
    return dict(
        id=hsm.id,
        name=hsm.name,
        hsm_uri=hsm.properties.hsm_uri,
        location=hsm.location,
        tags=hsm.tags,
        identity=hsm.identity and hsm.identity.as_dict() or None,
        enable_soft_delete=hsm.properties.enable_soft_delete,
        soft_delete_retention_in_days=hsm.properties.soft_delete_retention_in_days
        if hsm.properties.soft_delete_retention_in_days else 90,
        enable_purge_protection=hsm.properties.enable_purge_protection
        if hsm.properties.enable_purge_protection else False,
        sku=dict(
            family=hsm.sku.family,
            name=hsm.sku.name
        )
    )


class AzureRMKeyVaultInfo(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(type='str'),
            name=dict(type='str'),
            hsm_name=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.resource_group = None
        self.name = None
        self.hsm_name = None
        self.tags = None

        self.results = dict(changed=False)
        self._client = None

        super(AzureRMKeyVaultInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                  supports_check_mode=True,
                                                  supports_tags=False,
                                                  facts_module=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])

        self.module.deprecate(
            "The 'keys' return key in 'permissions' is deprecated. Use 'key_permissions' instead.",
            date="2027-05-01",
            collection_name="azure.azcollection",
        )

        self._client = self.get_mgmt_svc_client(KeyVaultManagementClient,
                                                base_url=self._cloud_environment.endpoints.resource_manager,
                                                api_version="2026-02-01")

        if self.name:
            if self.resource_group:
                self.results['keyvaults'] = self.get_by_vault_name()
            else:
                self.fail("resource_group is required when filtering by name")
        elif self.hsm_name:
            if self.resource_group:
                self.results['hsms'] = self.get_by_hsm_name()
            else:
                self.fail("resource_group is required when filtering by hsm_name")
        elif self.resource_group:
            self.results['keyvaults'] = self.list_vault_by_resource_group()
            self.results['hsms'] = self.list_hsm_by_resource_group()
        else:
            self.results['keyvaults'] = self.list_vault()
            self.results['hsms'] = self.list_hsm()

        return self.results

    def get_by_hsm_name(self):
        '''
        Gets the properties of this specified hsm.

        :return: deserialized hsm state dictionary
        '''
        self.log("Get the hsm {0}".format(self.hsm_name))
        results = []
        try:
            response = self._client.managed_hsms.get(resource_group_name=self.resource_group, name=self.hsm_name)
            self.log("Response : {0}".format(response))
            if response and self.has_tags(response.tags, self.tags):
                results.append(hsm_to_dict(response))
        except ResourceNotFoundError as e:
            self.log("Did not find the hsm {0}: {1}".format(self.hsm_name, str(e)))
        return results

    def get_by_vault_name(self):
        '''
        Gets the properties of this specified key vault.

        :return: deserialized key vault state dictionary
        '''
        self.log("Get the key vault {0}".format(self.name))
        results = []
        try:
            response = self._client.vaults.get(resource_group_name=self.resource_group, vault_name=self.name)
            self.log("Response : {0}".format(response))
            if response and self.has_tags(response.tags, self.tags):
                results.append(keyvault_to_dict(response))
        except ResourceNotFoundError as e:
            self.log("Did not find the key vault {0}: {1}".format(self.name, str(e)))
        return results

    def list_hsm_by_resource_group(self):
        '''
        Lists the properties of hsms in specific resource group.

        :return: deserialized hsm state dictionary
        '''
        self.log("Get the hsms in resource group {0}".format(self.resource_group))

        results = []
        try:
            response = list(self._client.managed_hsms.list_by_resource_group(resource_group_name=self.resource_group))
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    if self.has_tags(item.tags, self.tags):
                        results.append(hsm_to_dict(item))
        except Exception as e:
            self.log("Did not find hsms in resource group {0} : {1}.".format(self.resource_group, str(e)))
        return results

    def list_vault_by_resource_group(self):
        '''
        Lists the properties of key vaults in specific resource group.

        :return: deserialized key vaults state dictionary
        '''
        self.log("Get the key vaults in resource group {0}".format(self.resource_group))

        results = []
        try:
            response = list(self._client.vaults.list_by_resource_group(resource_group_name=self.resource_group))
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    if self.has_tags(item.tags, self.tags):
                        results.append(keyvault_to_dict(item))
        except Exception as e:
            self.log("Did not find key vaults in resource group {0} : {1}.".format(self.resource_group, str(e)))
        return results

    def list_vault(self):
        '''
        Lists the properties of key vaults in specific subscription.

        :return: deserialized key vaults state dictionary
        '''
        self.log("Get the key vaults in current subscription")

        results = []
        try:
            response = list(self._client.vaults.list())
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    if self.has_tags(item.tags, self.tags):
                        source_id = item.id.split('/')
                        results.append(keyvault_to_dict(self._client.vaults.get(source_id[4], source_id[8])))
        except Exception as e:
            self.log("Did not find key vault in current subscription {0}.".format(str(e)))
        return results

    def list_hsm(self):
        '''
        Lists the properties of hsms in specific subscription.

        :return: deserialized hsms state dictionary
        '''
        self.log("Get the hsms in current subscription")

        results = []
        try:
            response = list(self._client.managed_hsms.list_by_subscription())
            self.log("Response : {0}".format(response))

            if response:
                for item in response:
                    if self.has_tags(item.tags, self.tags):
                        source_id = item.id.split('/')
                        results.append(hsm_to_dict(self._client.managed_hsms.get(source_id[4], source_id[8])))
        except Exception as e:
            self.log("Did not find hsm in current subscription {0}.".format(str(e)))
        return results


def main():
    """Main execution"""
    AzureRMKeyVaultInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2018 Yuwei Zhou, <yuwzho@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_aksversion_info

version_added: "0.1.2"

short_description: Get available kubernetes versions supported by Azure Kubernetes Service

description:
    - Get available kubernetes versions supported by Azure Kubernetes Service.

options:
    location:
        description:
            - Get the versions available for creating a managed Kubernetes cluster.
        required: true
        type: str
    version:
        description:
            - Get the upgrade versions available for a managed Kubernetes cluster version.
        type: str
    allow_preview:
        description:
            - Whether Kubernetes version is currently in preview.
            - If I(allow_preview=True), returns the current preview status of the current Kubernetes version.
            - If I(allow_preview=False), returns the Kubernetes version in a non-current preview state.
            - If no set C(allow_preview), returns all the avaiable Kubernetes version.
        type: bool

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Yuwei Zhou (@yuwzho)
'''

EXAMPLES = '''
- name: Get current preview versions for AKS in location eastus
  azure_rm_aksversion_info:
    location: eastus
    allow_preview: true
- name: Get available versions for AKS in location eastus
  azure_rm_aksversion_info:
    location: eastus
- name: Get  available versions an AKS can be upgrade to
  azure_rm_aksversion_info:
    location: eastis
    version: 1.11.6
'''

RETURN = '''
azure_aks_versions:
    description: List of supported kubernetes versions.
    returned: always
    type: list
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMAKSVersion(AzureRMModuleBase):

    def __init__(self):

        self.module_args = dict(
            location=dict(type='str', required=True),
            version=dict(type='str'),
            allow_preview=dict(type='bool')
        )

        self.results = dict(
            changed=False,
            azure_aks_versions=[]
        )

        self.location = None
        self.version = None
        self.allow_preview = None

        super(AzureRMAKSVersion, self).__init__(
            derived_arg_spec=self.module_args,
            supports_check_mode=True,
            supports_tags=False,
            facts_module=True
        )

    def exec_module(self, **kwargs):

        for key in self.module_args:
            setattr(self, key, kwargs[key])

        self.results['azure_aks_versions'] = self.get_all_versions(self.location, self.version)

        return self.results

    def get_all_versions(self, location, version):
        '''
        Get all kubernetes version supported by AKS
        :return: ordered version list
        '''
        try:
            result = dict()
            response = self.containerservice_client.container_services.list_orchestrators(self.location, resource_type='managedClusters')
            orchestrators = response.orchestrators
            for item in orchestrators:
                if self.allow_preview is None:
                    result[item.orchestrator_version] = [x.orchestrator_version for x in item.upgrades] if item.upgrades else []
                elif self.allow_preview:
                    if item.is_preview:
                        result[item.orchestrator_version] = [x.orchestrator_version for x in item.upgrades] if item.upgrades else []
                else:
                    if not item.is_preview:
                        result[item.orchestrator_version] = [x.orchestrator_version for x in item.upgrades] if item.upgrades else []
            if version:
                return result.get(version) or []
            else:
                keys = list(result.keys())
                keys.sort()
                return keys
        except Exception as exc:
            self.fail('Error when getting AKS supported kubernetes version list for location {0} - {1}'.format(self.location, exc.message or str(exc)))


def main():
    """Main module execution code path"""

    AzureRMAKSVersion()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_wireless_air_marshal_rules
short_description: Resource module for networks _wireless _air _marshal _rules
description:
  - Manage operations create, update and delete of the resource networks _wireless
    _air _marshal _rules.
  - Creates a new rule.
  - Delete an Air Marshal rule.
  - Update a rule.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  match:
    description: Object describing the rule specification.
    suboptions:
      string:
        description: The string used to match.
        type: str
      type:
        description: The type of match.
        type: str
    type: dict
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  ruleId:
    description: RuleId path parameter. Rule ID.
    type: str
  type:
    description: Indicates if this rule will allow, block, or alert.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for wireless createNetworkWirelessAirMarshalRule
    description: Complete reference of the createNetworkWirelessAirMarshalRule API.
    link: https://developer.cisco.com/meraki/api-v1/#!create-network-wireless-air-marshal-rule
  - name: Cisco Meraki documentation for wireless deleteNetworkWirelessAirMarshalRule
    description: Complete reference of the deleteNetworkWirelessAirMarshalRule API.
    link: https://developer.cisco.com/meraki/api-v1/#!delete-network-wireless-air-marshal-rule
  - name: Cisco Meraki documentation for wireless updateNetworkWirelessAirMarshalRule
    description: Complete reference of the updateNetworkWirelessAirMarshalRule API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-wireless-air-marshal-rule
notes:
  - SDK Method used are
    wireless.Wireless.create_network_wireless_air_marshal_rule,
    wireless.Wireless.delete_network_wireless_air_marshal_rule,
    wireless.Wireless.update_network_wireless_air_marshal_rule,
  - Paths used are
    post /networks/{networkId}/wireless/airMarshal/rules,
    delete /networks/{networkId}/wireless/airMarshal/rules/{ruleId},
    put /networks/{networkId}/wireless/airMarshal/rules/{ruleId},
"""

EXAMPLES = r"""
- name: Create
  cisco.meraki.networks_wireless_air_marshal_rules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    match:
      string: 00:11:22:33:44:55
      type: bssid
    networkId: string
    type: allow
- name: Update by id
  cisco.meraki.networks_wireless_air_marshal_rules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    match:
      string: 00:11:22:33:44:55
      type: bssid
    networkId: string
    ruleId: string
    type: allow
- name: Delete by id
  cisco.meraki.networks_wireless_air_marshal_rules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: absent
    networkId: string
    ruleId: string
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "network": {
        "id": "string",
        "name": "string"
      },
      "ruleId": "string",
      "type": "string",
      "updatedAt": "string",
      "createdAt": "string",
      "match": {
        "string": "string",
        "type": "string"
      }
    }
"""



# (c) 2020 Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
    author:
      - Ansible Networking Team (@ansible-network)
    name: libssh
    short_description: Run tasks using libssh for ssh connection
    description:
        - Use the ansible-pylibssh python bindings to connect to targets
        - The python bindings use libssh C library (https://www.libssh.org/) to connect to targets
        - This plugin borrows a lot of settings from the ssh plugin as they both cover the same protocol.
    version_added: 1.1.0
    options:
      remote_addr:
        description:
            - Address of the remote target
        type: string
        default: inventory_hostname
        vars:
            - name: inventory_hostname
            - name: ansible_host
            - name: ansible_ssh_host
            - name: ansible_libssh_host
      remote_user:
        description:
            - User to login/authenticate as
            - Can be set from the CLI via the C(--user) or C(-u) options.
        type: string
        vars:
            - name: ansible_user
            - name: ansible_ssh_user
            - name: ansible_libssh_user
        env:
            - name: ANSIBLE_REMOTE_USER
            - name: ANSIBLE_LIBSSH_REMOTE_USER
        ini:
            - section: defaults
              key: remote_user
            - section: libssh_connection
              key: remote_user
      password:
        description:
          - Secret used to either login the ssh server or as a passphrase for ssh keys that require it
          - Can be set from the CLI via the C(--ask-pass) option.
        type: string
        vars:
            - name: ansible_password
            - name: ansible_ssh_pass
            - name: ansible_ssh_password
            - name: ansible_libssh_pass
            - name: ansible_libssh_password
      password_prompt:
        description:
          - Text to match when using keyboard-interactive authentication to determine if the prompt is
            for the password.
          - Requires ansible-pylibssh version >= 1.0.0
        type: string
        vars:
          - name: ansible_libssh_password_prompt
        version_added: 3.1.0
      private_key_passphrase:
        description:
          - Passphrase used to unlock the private key specified by the C(ansible_private_key_file) attribute.
          - This is required if the private key is encrypted with a passphrase.
        type: string
        vars:
            - name: ansible_private_key_password
            - name: ansible_private_key_passphrase
      host_key_auto_add:
        description: 'TODO: write it'
        env: [{name: ANSIBLE_LIBSSH_HOST_KEY_AUTO_ADD}]
        ini:
          - {key: host_key_auto_add, section: libssh_connection}
        type: boolean
      look_for_keys:
        default: True
        description: 'TODO: write it'
        env: [{name: ANSIBLE_LIBSSH_LOOK_FOR_KEYS}]
        ini:
        - {key: look_for_keys, section: libssh_connection}
        type: boolean
      proxy_command:
        default: ''
        description:
            - Proxy information for running the connection via a jumphost.
            - Also this plugin will scan 'ssh_args', 'ssh_extra_args' and 'ssh_common_args' from the 'ssh' plugin settings for proxy information if set.
        type: string
        env:
          - name: ANSIBLE_LIBSSH_PROXY_COMMAND
        ini:
          - {key: proxy_command, section: libssh_connection}
        vars:
          - name: ansible_paramiko_proxy_command
          - name: ansible_libssh_proxy_command
      pty:
        default: True
        description: 'TODO: write it'
        env:
          - name: ANSIBLE_LIBSSH_PTY
        ini:
          - section: libssh_connection
            key: pty
        type: boolean
      publickey_accepted_algorithms:
        default: ''
        description:
            - List of algorithms to forward to SSH_OPTIONS_PUBLICKEY_ACCEPTED_TYPES.
        type: str
        env:
          - name: ANSIBLE_LIBSSH_PUBLICKEY_ALGORITHMS
        ini:
          - {key: publickey_algorithms, section: libssh_connection}
        vars:
          - name: ansible_libssh_publickey_algorithms
      hostkeys:
        default: ''
        description: Set the preferred server host key types as a comma-separated list (e.g., ssh-rsa,ssh-dss,ecdh-sha2-nistp256).
        type: str
        env:
          - name: ANSIBLE_LIBSSH_HOSTKEYS
        ini:
          - {key: hostkeys, section: libssh_connection}
        vars:
          - name: ansible_libssh_hostkeys
      key_exchange_algorithms:
        description:
          - Set the key exchange method as a comma-separated list (e.g., "ecdh-sha2-nistp256,diffie-hellman-group14-sha1,diffie-hellman-group1-sha1").
          - The list can be prepended by +,-,^ which will append, remove or move to the beginning (prioritizing) of the default list respectively.
            Giving an empty list after + and ^ will cause error.
        type: str
        env:
          - name: ANSIBLE_LIBSSH_KEY_EXCHANGE_ALGORITHMS
        ini:
          - key: key_exchange_algorithms
            section: libssh_connection
        vars:
          - name: ansible_libssh_key_exchange_algorithms
      host_key_checking:
        description: 'Set this to "False" if you want to avoid host key checking by the underlying tools Ansible uses to connect to the host'
        type: boolean
        default: True
        env:
          - name: ANSIBLE_HOST_KEY_CHECKING
          - name: ANSIBLE_SSH_HOST_KEY_CHECKING
          - name: ANSIBLE_LIBSSH_HOST_KEY_CHECKING
        ini:
          - section: defaults
            key: host_key_checking
          - section: libssh_connection
            key: host_key_checking
        vars:
          - name: ansible_host_key_checking
          - name: ansible_ssh_host_key_checking
          - name: ansible_libssh_host_key_checking
      use_persistent_connections:
        description: 'Toggles the use of persistence for connections'
        type: boolean
        default: False
        env:
          - name: ANSIBLE_USE_PERSISTENT_CONNECTIONS
        ini:
          - section: defaults
            key: use_persistent_connections
      ssh_args:
          version_added: 3.2.0
          description:
           - Arguments to pass to all ssh CLI tools.
           - ProxyCommand is the only supported argument.
           - This option is deprecated in favor of I(proxy_command) and will be removed
             in a release after 2026-01-01.
          type: string
          ini:
              - section: 'ssh_connection'
                key: 'ssh_args'
          env:
              - name: ANSIBLE_SSH_ARGS
          vars:
              - name: ansible_ssh_args
          cli:
              - name: ssh_args
      ssh_common_args:
          version_added: 3.2.0
          description:
           - Common extra arguments for all ssh CLI tools.
           - ProxyCommand is the only supported argument.
           - This option is deprecated in favor of I(proxy_command) and will be removed
             in a release after 2026-01-01.
          type: string
          ini:
              - section: 'ssh_connection'
                key: 'ssh_common_args'
          env:
              - name: ANSIBLE_SSH_COMMON_ARGS
          vars:
              - name: ansible_ssh_common_args
          cli:
              - name: ssh_common_args
      ssh_extra_args:
          version_added: 3.2.0
          description:
           - Extra arguments exclusive to the 'ssh' CLI tool.
           - ProxyCommand is the only supported argument.
           - This option is deprecated in favor of I(proxy_command) and will be removed
             in a release after 2026-01-01.
          type: string
          vars:
              - name: ansible_ssh_extra_args
          env:
            - name: ANSIBLE_SSH_EXTRA_ARGS
          ini:
            - key: ssh_extra_args
              section: ssh_connection
          cli:
            - name: ssh_extra_args
      config_file:
        version_added: 5.1.0
        description: Alternate SSH config file location
        type: path
        env:
          - name: ANSIBLE_LIBSSH_CONFIG_FILE
        ini:
          - section: libssh_connection
            key: config_file
        vars:
          - name: ansible_libssh_config_file
# TODO:
#timeout=self._play_context.timeout,
"""
import logging
import os
import re
import socket

from ansible import constants as C
from ansible.errors import AnsibleConnectionFailure, AnsibleError, AnsibleFileNotFound
from ansible.module_utils.basic import missing_required_lib
from ansible.module_utils.common.text.converters import to_bytes, to_native, to_text
from ansible.plugins.connection import ConnectionBase
from ansible.utils.display import Display

from ansible_collections.ansible.netcommon.plugins.plugin_utils.version import Version


display = Display()

try:
    from pylibsshext import __version__ as PYLIBSSH_VERSION
    from pylibsshext.errors import LibsshSCPException, LibsshSessionException
    from pylibsshext.session import Session

    HAS_PYLIBSSH = True
except ImportError:
    HAS_PYLIBSSH = False


AUTHENTICITY_MSG = """
libssh: The authenticity of host '%s' can't be established due to '%s'.
The %s key fingerprint is %s.
Are you sure you want to continue connecting (yes/no)?
"""

# SSH Options Regex
SETTINGS_REGEX = re.compile(r"(\w+)(?:\s*=\s*|\s+)(.+)")


class MyAddPolicy(object):
    """
    Based on AutoAddPolicy in paramiko so we can determine when keys are added

    and also prompt for input.

    Policy for automatically adding the hostname and new host key to the
    local L{HostKeys} object, and saving it.  This is used by L{SSHClient}.
    """

    def __init__(self, connection):
        self.connection = connection
        self._options = connection._options

    def missing_host_key(self, session, hostname, username, key_type, fingerprint, message):
        if all(
            (
                self._options["host_key_checking"],
                not self._options["host_key_auto_add"],
            )
        ):
            if (
                self.connection.get_option("use_persistent_connections")
                or self.connection.force_persistence
            ):
                # don't print the prompt string since the user cannot respond
                # to the question anyway
                raise AnsibleError(
                    AUTHENTICITY_MSG.rsplit("\n", 2)[0] % (hostname, message, key_type, fingerprint)
                )

            inp = to_text(
                display.prompt_until(
                    AUTHENTICITY_MSG % (hostname, message, key_type, fingerprint), private=False
                ),
                errors="surrogate_or_strict",
            )

            self.connection.connection_unlock()
            if inp not in ["yes", "y", ""]:
                raise AnsibleError("host connection rejected by user")

        session.hostkey_auto_add(username)

        # host keys are actually saved in close() function below
        # in order to control ordering.


# keep connection objects on a per host basis to avoid repeated attempts to reconnect
SSH_CONNECTION_CACHE = {}
SFTP_CONNECTION_CACHE = {}


class Connection(ConnectionBase):
    """SSH based connections with Libssh"""

    transport = "ansible.netcommon.libssh"
    _log_channel = None

    def _cache_key(self):
        return "%s__%s__" % (
            self._play_context.remote_addr,
            self._play_context.remote_user,
        )

    def _connect(self):
        cache_key = self._cache_key()
        if cache_key in SSH_CONNECTION_CACHE:
            self.ssh = SSH_CONNECTION_CACHE[cache_key]
        else:
            self.ssh = SSH_CONNECTION_CACHE[cache_key] = self._connect_uncached()
        return self

    def _set_log_channel(self, name):
        self._log_channel = name

    @staticmethod
    def _pylibssh_handler_log_level():
        verbosity = display.verbosity
        if verbosity >= 3:
            return logging.DEBUG
        if verbosity >= 1:
            return logging.INFO
        return logging.WARNING

    def _pylibssh_resolve_log_path(self, host):
        logpath = getattr(C, "DEFAULT_LOG_PATH", None)
        display.vvvv(
            "libssh log handler: DEFAULT_LOG_PATH=%s" % repr(logpath),
            host=host,
        )
        if not logpath:
            display.vvvv("libssh log handler: skipped (no DEFAULT_LOG_PATH set)", host=host)
            return None
        if not os.path.isabs(logpath):
            display.vvvv(
                "libssh log handler: skipped (log_path not absolute: %s)" % logpath,
                host=host,
            )
            return None
        logpath = os.path.expanduser(logpath)
        if os.path.exists(logpath):
            if not os.access(logpath, os.W_OK):
                display.vvvv(
                    "libssh log handler: skipped (log file not writable: %s)" % logpath,
                    host=host,
                )
                return None
            return logpath
        parent = os.path.dirname(logpath)
        if parent and not os.path.isdir(parent):
            display.vvvv(
                "libssh log handler: skipped (parent not a directory: %s)" % parent,
                host=host,
            )
            return None
        if parent and not os.access(parent, os.W_OK):
            display.vvvv(
                "libssh log handler: skipped (parent not writable: %s)" % parent,
                host=host,
            )
            return None
        return logpath

    def _pylibssh_attach_log_handler(self, logpath, log_level, host):
        pylibssh_log = logging.getLogger("ansible-pylibssh")
        for h in pylibssh_log.handlers:
            if getattr(h, "baseFilename", None) == logpath:
                pylibssh_log.setLevel(log_level)
                h.setLevel(log_level)
                display.vvvv(
                    "libssh log handler: already attached to %s (level=%s)" % (logpath, log_level),
                    host=host,
                )
                return
        handler = logging.FileHandler(logpath, mode="a", encoding="utf-8")
        handler.setLevel(log_level)
        handler.setFormatter(
            logging.Formatter("%(asctime)s p=%(process)d n=%(name)s %(levelname)s| %(message)s")
        )
        pylibssh_log.addHandler(handler)
        pylibssh_log.setLevel(log_level)
        display.vvvv(
            "libssh log handler: added FileHandler for ansible-pylibssh -> %s (level=%s)"
            % (logpath, log_level),
            host=host,
        )

    def _ensure_pylibssh_log_handler(self, host=None):
        """Route ansible-pylibssh (libssh) logs to Ansible log_path when set.

        Handler and logger levels follow display.verbosity using Python standard
        logging levels: 0 -> WARNING, 1-2 -> INFO, 3+ -> DEBUG.
        """
        log_level = self._pylibssh_handler_log_level()
        logpath = self._pylibssh_resolve_log_path(host)
        if logpath is None:
            return
        self._pylibssh_attach_log_handler(logpath, log_level, host)

    def _get_proxy_command(self, port=22):
        proxy_command = None
        # Parse ansible_ssh_common_args, specifically looking for ProxyCommand
        ssh_args = [
            self.get_option("ssh_extra_args") or "",
            self.get_option("ssh_common_args") or "",
            self.get_option("ssh_args") or "",
        ]

        if any(ssh_args):
            display.warning(
                "The ssh_*_args options are deprecated and will be removed in a release after 2026-01-01. Please use the proxy_command option instead."
            )
            args = self._split_ssh_args(" ".join(ssh_args))
            for i, arg in enumerate(args):
                if arg.lower() == "proxycommand":
                    # _split_ssh_args split ProxyCommand from the command itself
                    proxy_command = args[i + 1]
                else:
                    # ProxyCommand and the command itself are a single string
                    match = SETTINGS_REGEX.match(arg)
                    if match:
                        if match.group(1).lower() == "proxycommand":
                            proxy_command = match.group(2)

                if proxy_command:
                    break

        proxy_command = proxy_command or self.get_option("proxy_command")

        if proxy_command:
            replacers = {
                "%h": self._play_context.remote_addr,
                "%p": port,
                "%r": self._play_context.remote_user,
            }
            for find, replace in replacers.items():
                proxy_command = proxy_command.replace(find, str(replace))

        return proxy_command

    def _connect_uncached(self):
        """activates the connection object"""

        if not HAS_PYLIBSSH:
            raise AnsibleError(missing_required_lib("ansible-pylibssh"))
        display.vvv(
            "USING PYLIBSSH VERSION %s" % PYLIBSSH_VERSION,
            host=self._play_context.remote_addr,
        )

        ssh_connect_kwargs = {}

        remote_user = self.get_option("remote_user")
        remote_addr = self.get_option("remote_addr")
        port = self._play_context.port or 22
        display.vvv(
            "ESTABLISH LIBSSH CONNECTION FOR USER: %s on PORT %s TO %s"
            % (remote_user, port, remote_addr),
            host=remote_addr,
        )

        self.ssh = Session()
        self._ensure_pylibssh_log_handler(remote_addr)

        self.keyfile = os.path.expanduser("~/.ssh/known_hosts")

        proxy_command = self._get_proxy_command(port)

        try:
            private_key = None
            if self._play_context.private_key_file:
                with open(os.path.expanduser(self._play_context.private_key_file)) as fp:
                    b_content = fp.read()
                    private_key = to_bytes(b_content, errors="surrogate_or_strict")

            if proxy_command:
                ssh_connect_kwargs["proxycommand"] = proxy_command

            if self.get_option("config_file"):
                ssh_connect_kwargs["config_file"] = self.get_option("config_file")

            if self.get_option("password_prompt") and (Version(PYLIBSSH_VERSION) < "1.0.0"):
                raise AnsibleError(
                    "Configuring password prompt is not supported in ansible-pylibssh version %s. "
                    "Please upgrade to ansible-pylibssh 1.0.0 or newer." % PYLIBSSH_VERSION
                )

            if self.get_option("publickey_accepted_algorithms"):
                ssh_connect_kwargs["publickey_accepted_algorithms"] = self.get_option(
                    "publickey_accepted_algorithms"
                )

            if self.get_option("hostkeys"):
                ssh_connect_kwargs["hostkeys"] = self.get_option("hostkeys")

            if self.get_option("key_exchange_algorithms"):
                ssh_connect_kwargs["key_exchange_algorithms"] = self.get_option(
                    "key_exchange_algorithms"
                )

            self.ssh.set_missing_host_key_policy(MyAddPolicy(self))

            self.ssh.connect(
                host=remote_addr.lower(),
                user=remote_user,
                look_for_keys=self.get_option("look_for_keys"),
                host_key_checking=self.get_option("host_key_checking"),
                password=self.get_option("password"),
                password_prompt=self.get_option("password_prompt"),
                private_key=private_key,
                private_key_password=self.get_option("private_key_passphrase"),
                timeout=self._play_context.timeout,
                port=port,
                **ssh_connect_kwargs,
            )
        except LibsshSessionException as e:
            msg = "ssh connection failed: " + to_text(e)
            raise AnsibleConnectionFailure(msg)
        except Exception as e:
            raise AnsibleConnectionFailure(to_text(e))

        display.vvv("ssh connection is OK: " + str(self.ssh))
        return self.ssh

    def exec_command(self, cmd, in_data=None, sudoable=True):
        """run a command on the remote host"""

        super(Connection, self).exec_command(cmd, in_data=in_data, sudoable=sudoable)

        if in_data:
            raise AnsibleError(
                "Internal Error: this module does not support optimized module pipelining"
            )

        bufsize = 4096

        try:
            self.chan = self.ssh.new_channel()
        except Exception as e:
            text_e = to_text(e)
            msg = "Failed to open session"
            if text_e:
                msg += ": %s" % text_e
            raise AnsibleConnectionFailure(to_native(msg))

        # sudo usually requires a PTY (cf. requiretty option), therefore
        # we give it one by default (pty=True in ansible.cfg), and we try
        # to initialise from the calling environment when sudoable is enabled
        if self.get_option("pty") and sudoable:
            self.chan.request_shell()

        display.vvv("EXEC %s" % cmd, host=self._play_context.remote_addr)

        cmd = to_bytes(cmd, errors="surrogate_or_strict")

        result = None
        no_prompt_out = b""
        no_prompt_err = b""
        become_output = b""
        out = b""
        err = b""

        try:
            if self.become and self.become.expect_prompt():
                passprompt = False
                become_sucess = False
                self.chan.sendall(cmd)

                while not (become_sucess or passprompt):
                    display.debug("Waiting for Privilege Escalation input")
                    self.chan.poll(timeout=self._play_context.timeout)
                    chunk = self.chan.recv(bufsize)
                    display.debug("chunk is: %s" % chunk)

                    if not chunk:
                        if b"unknown user" in become_output:
                            n_become_user = to_native(
                                self.become.get_option(
                                    "become_user",
                                    playcontext=self._play_context,
                                )
                            )
                            raise AnsibleError("user %s does not exist" % n_become_user)
                        else:
                            break
                            # raise AnsibleError('ssh connection closed waiting for password prompt')
                    become_output += chunk
                    # need to check every line because we might get lectured
                    # and we might get the middle of a line in a chunk
                    for line in become_output.splitlines(True):
                        if self.become.check_success(line):
                            become_sucess = True
                            break
                        if self.become.check_password_prompt(line):
                            passprompt = True
                            break
                if passprompt:
                    if self.become:
                        become_pass = self.become.get_option(
                            "become_pass", playcontext=self._play_context
                        )
                        self.chan.sendall(
                            to_bytes(become_pass, errors="surrogate_or_strict") + b"\n"
                        )
                    else:
                        raise AnsibleError("A password is required but none was supplied")
                else:
                    no_prompt_out += become_output
                    no_prompt_err += become_output
            else:
                result = self.chan.exec_command(to_text(cmd, errors="surrogate_or_strict"))
        except socket.timeout:
            raise AnsibleError("ssh timed out waiting for privilege escalation.\n" + become_output)

        if result:
            rc = result.returncode
            out = result.stdout
            err = result.stderr
        else:
            rc = self.chan.get_channel_exit_status()
        return rc, out, err

    def put_file(self, in_path, out_path, proto="sftp"):
        """transfer a file from local to remote"""

        super(Connection, self).put_file(in_path, out_path)

        display.vvv(
            "PUT %s TO %s" % (in_path, out_path),
            host=self._play_context.remote_addr,
        )

        if not os.path.exists(to_bytes(in_path, errors="surrogate_or_strict")):
            raise AnsibleFileNotFound("file or module does not exist: %s" % in_path)

        self._connect()
        if proto == "sftp":
            try:
                self.sftp = self.ssh.sftp()
            except Exception as e:
                raise AnsibleError("failed to open a SFTP connection (%s)" % e)

            try:
                self.sftp.put(
                    to_bytes(in_path, errors="surrogate_or_strict"),
                    to_bytes(out_path, errors="surrogate_or_strict"),
                )
            except IOError:
                raise AnsibleError("failed to transfer file to %s" % out_path)
        elif proto == "scp":
            scp = self.ssh.scp()
            try:
                scp.put(in_path, out_path)
            except LibsshSCPException as exc:
                raise AnsibleError("Error transferring file to %s: %s" % (out_path, to_text(exc)))
        else:
            raise AnsibleError("Don't know how to transfer file over protocol %s" % proto)

    def _connect_sftp(self):
        cache_key = "%s__%s__" % (
            self._play_context.remote_addr,
            self._play_context.remote_user,
        )
        if cache_key in SFTP_CONNECTION_CACHE:
            return SFTP_CONNECTION_CACHE[cache_key]
        else:
            result = SFTP_CONNECTION_CACHE[cache_key] = self._connect().ssh.sftp()
            return result

    def _invalidate_ssh_session_after_scp_get_failure(self):
        """Drop cached SSH/SFTP and close session after failed SCP get (device quirks)."""
        cache_key = self._cache_key()
        SSH_CONNECTION_CACHE.pop(cache_key, None)
        SFTP_CONNECTION_CACHE.pop(cache_key, None)
        if hasattr(self, "ssh") and self.ssh is not None:
            try:
                self.ssh.close()
            except Exception:
                pass
            self.ssh = None

    def _fetch_file_via_sftp(self, in_path, out_path):
        try:
            self.sftp = self._connect_sftp()
        except Exception as e:
            raise AnsibleError("failed to open a SFTP connection (%s)" % to_native(e))

        try:
            self.sftp.get(
                to_bytes(in_path, errors="surrogate_or_strict"),
                to_bytes(out_path, errors="surrogate_or_strict"),
            )
        except IOError:
            raise AnsibleError("failed to transfer file from %s" % in_path)

    def _fetch_file_via_scp(self, in_path, out_path):
        scp = self.ssh.scp()
        try:
            # this abruptly closes the connection when
            # scp.get fails only when the file is not there
            # it works fine if the file is actually present
            scp.get(in_path, out_path)
        except LibsshSCPException as exc:
            # Invalidate cached connection so next operation (e.g. put_file)
            # uses a fresh connection; some devices misbehave when reusing
            # a connection that had a failed SCP get.
            self._invalidate_ssh_session_after_scp_get_failure()
            raise AnsibleError("Error transferring file from %s: %s" % (in_path, to_text(exc)))

    def fetch_file(self, in_path, out_path, proto="sftp"):
        """save a remote file to the specified path"""

        super(Connection, self).fetch_file(in_path, out_path)

        display.vvv(
            "FETCH %s TO %s" % (in_path, out_path),
            host=self._play_context.remote_addr,
        )

        if proto == "sftp":
            self._fetch_file_via_sftp(in_path, out_path)
        elif proto == "scp":
            self._fetch_file_via_scp(in_path, out_path)
        else:
            raise AnsibleError("Don't know how to transfer file over protocol %s" % proto)

    def reset(self):
        self.close()
        self._connect()

    def close(self):
        """terminate the connection"""

        cache_key = self._cache_key()
        SSH_CONNECTION_CACHE.pop(cache_key, None)
        SFTP_CONNECTION_CACHE.pop(cache_key, None)

        if hasattr(self, "sftp"):
            if self.sftp is not None:
                self.sftp.close()

        if hasattr(self, "chan"):
            if self.chan is not None:
                self.chan.close()

        self.ssh.close()
        self._connected = False



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_storageblob_info
short_description: Get or list the containers blob facts
version_added: "3.0.0"
description:
    - Get or list the blobs under the specified container.
options:
    auth_mode:
        description:
            - The mode in which to run the command. C(login) mode will directly use your login credentials for the authentication.
            - The legacy C(key) mode will attempt to query for an account key if no authentication parameters for the account are provided.
            - Can also be set via the environment variable C(AZURE_STORAGE_AUTH_MODE).
        default: key
        type: str
        choices:
            - key
            - login
    storage_account_name:
        description:
            - Name of the storage account to use.
        required: true
        type: str
        aliases:
            - account_name
            - storage_account
    blob_name:
        description:
            - Name of a blob object within the container.
        aliases:
            - blob
        type: str
    container_name:
        description:
            - Name of a blob container within the storage account.
        required: true
        type: str
        aliases:
            - container
    resource_group:
        description:
            - Name of the resource group to use.
        required: true
        type: str
        aliases:
            - resource_group_name
    name_starts_with:
        description:
            - Filters the results to return only blobs whose names begin with the specified prefix.
        type: str
    include:
        description:
            - Specifies one or more additional datasets to include in the response.
        type: list
        elements: str
        choices:
            - snapshots
            - metadata
            - uncommittedblobs
            - copy
            - deleted
            - deletedwithversions
            - tags
            - versions
            - immutabilitypolicy
            - legalhold
extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Get the blob facts by name
  azure_rm_storageblob_info:
    resource_group: "{{ resource_group }}"
    account_name: "{{ storage_account }}"
    container_name: my-blobs
    blob_name: blobname01

- name: List the blob facts in specify container
  azure_rm_storageblob_info:
    resource_group: "{{ resource_group }}"
    account_name: "{{ storage_account }}"
    container_name: my-blobs
'''

RETURN = '''
blob:
    description:
        - Facts about the current state of the blob.
    returned: when a blob is operated on
    type: dict
    sample: {
        "content_length": 136532,
        "content_settings": {
            "cache_control": null,
            "content_disposition": null,
            "content_encoding": null,
            "content_language": null,
            "content_md5": null,
            "content_type": "application/image"
        },
        "last_modified": "20-11-2024 22:08:25 +0000",
        "name": "graylog.png",
        'metadata': {'key1': 'value1'},
        "standard_blob_tier": "Hot",
        "tags": {},
        "type": "BlockBlob"
    }
'''


try:
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase
from ansible.module_utils.basic import env_fallback


class AzureRMStorageBlobInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            auth_mode=dict(
                type='str',
                choices=['key', 'login'],
                fallback=(env_fallback, ['AZURE_STORAGE_AUTH_MODE']),
                default="key"
            ),
            storage_account_name=dict(required=True, type='str', aliases=['account_name', 'storage_account']),
            blob_name=dict(type='str', aliases=['blob']),
            container_name=dict(required=True, type='str', aliases=['container']),
            resource_group=dict(required=True, type='str', aliases=['resource_group_name']),
            name_starts_with=dict(type='str'),
            include=dict(
                type='list',
                elements='str',
                choices=['snapshots', 'metadata', 'uncommittedblobs', 'copy', 'deleted',
                         'deletedwithversions', 'tags', 'versions', 'immutabilitypolicy', 'legalhold']
            ),
        )

        self.blob_service_client = None
        self.storage_account_name = None
        self.blob_name = None
        self.container_name = None
        self.resource_group = None
        self.name_starts_with = None
        self.include = None
        self.results = dict(
            changed=False,
            blob=dict()
        )

        super(AzureRMStorageBlobInfo, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                     supports_check_mode=True,
                                                     supports_tags=False)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self.blob_service_client = self.get_blob_service_client(self.resource_group, self.storage_account_name, self.auth_mode)
        if self.blob_name:
            response = self.get_blob()
        else:
            response = self.list_blob()
        self.results['blob'] = response
        return self.results

    def get_blob(self):
        response = None
        if self.blob_name:
            try:
                response = self.blob_service_client.get_blob_client(container=self.container_name, blob=self.blob_name).get_blob_properties()
            except ResourceNotFoundError:
                pass
        return self.format_blob(response) if response else None

    def list_blob(self):
        response = []
        try:
            client = self.blob_service_client.get_container_client(container=self.container_name)
            blobs = client.list_blobs(name_starts_with=self.name_starts_with, include=self.include)
            for blob in blobs:
                response.append(self.format_blob(blob))
        except Exception as exc:
            self.fail("Error list container blob {0} - {1}".format(self.container_name, str(exc)))
        return response

    def format_blob(self, blob):
        result = dict(
            name=blob["name"],
            tags=blob["metadata"],
            last_modified=blob["last_modified"].strftime('%d-%b-%Y %H:%M:%S %z'),
            type=blob["blob_type"],
            standard_blob_tier=blob.get('blob_tier'),
            metadata=blob.get('metadata'),
            content_length=blob["size"],
            content_settings=dict(
                content_type=blob["content_settings"]["content_type"],
                content_encoding=blob["content_settings"]["content_encoding"],
                content_language=blob["content_settings"]["content_language"],
                content_disposition=blob["content_settings"]["content_disposition"],
                cache_control=blob["content_settings"]["cache_control"],
                content_md5=blob["content_settings"]["content_md5"].hex() if blob["content_settings"]["content_md5"] else None,
            )
        )
        return result


def main():
    AzureRMStorageBlobInfo()


if __name__ == '__main__':
    main()
